// SPDX-FileCopyrightText: 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#include "compactlistview.h"
#include "widgets/multiselectlistview.h"

#include <QDebug>
#include <QDBusInterface>
#include <QDBusArgument>
#include <QDBusPendingReply>

#define COMPACTMODE 1

CompactListView::CompactListView(const QSize &normalSize, const QSize &compactModeSize, QWidget *parent)
    : MultiSelectListView(parent)
    , m_normalSize(normalSize)
    , m_compactModeSize(compactModeSize)
{
    setAccessibleName("CompactListView");
    QDBusConnection::sessionBus().connect("com.deepin.daemon.Appearance",
                                          "/com/deepin/daemon/Appearance",
                                          "org.freedesktop.DBus.Properties",
                                          "PropertiesChanged",
                                          "sa{sv}as",
                                          this, SLOT(handleAppearanceDbusSignal(QDBusMessage)));
    setAutoSizeForModeChanged(m_normalSize, m_compactModeSize);
}

void CompactListView::handleAppearanceDbusSignal(QDBusMessage message)
{
    QList<QVariant> arguments = message.arguments();
    if (3 != arguments.count())
        return;

    const QString &interfaceName = message.arguments().at(0).toString();
    if (interfaceName != "com.deepin.daemon.Appearance")
        return;

    const QVariantMap& changedProps = qdbus_cast<QVariantMap>(arguments.at(1).value<QDBusArgument>());
    for (const auto &key : changedProps.keys()) {
        if (key != "DTKSizeMode")
            continue;

        setItemSize(currentSizeMode() == COMPACTMODE ? m_compactModeSize : m_normalSize);
        break;
    }
}

int CompactListView::currentSizeMode() const
{
    QDBusInterface interface("com.deepin.daemon.Appearance",
                             "/com/deepin/daemon/Appearance",
                             "org.freedesktop.DBus.Properties",
                             QDBusConnection::sessionBus());
    const QDBusPendingReply<QDBusVariant> &reply = interface.call("Get", "com.deepin.daemon.Appearance", "DTKSizeMode");
    if (reply.isValid())
        return reply.value().variant().toInt();

    qWarning() << "call DTKSizeMode failed: " << reply.error().message();
    return -1;
}

void CompactListView::setAutoSizeForModeChanged(const QSize &normalSize, const QSize &compactModeSize)
{
    m_normalSize = normalSize;
    m_compactModeSize = compactModeSize;
    setItemSize(currentSizeMode() == COMPACTMODE ? m_compactModeSize : m_normalSize);
}

void CompactListView::setAutoSpacingForModeChanged(int normalSpace, int compactModeSpace)
{
    setSpacing(currentSizeMode() == COMPACTMODE ? compactModeSpace : normalSpace);
}
