// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "networkmodulewidget.h"
#include "window/utils.h"
#include "window/gsettingwatcher.h"
#include "widgets/nextpagewidget.h"
#include "widgets/settingsgroup.h"
#include "widgets/multiselectlistview.h"
#include "compactlistview.h"

#include <DStyleOption>

#include <QPointer>
#include <QVBoxLayout>
#include <QPushButton>
#include <QProcess>
#include <QDBusInterface>
#include <QDBusMessage>

#include <interface/frameproxyinterface.h>

#include <wireddevice.h>
#include <wirelessdevice.h>
#include <networkdevicebase.h>
#include <networkcontroller.h>
#include <proxycontroller.h>
#include <networkconst.h>
#include <hotspotcontroller.h>

#include <widgets/contentwidget.h>

#include <org_freedesktop_notifications.h>

using namespace dcc::widgets;
using namespace DCC_NAMESPACE;
using namespace dde::network;
using Notifications = org::freedesktop::Notifications;

static const int SectionRole = Dtk::UserRole + 1;
static const int DeviceRole = Dtk::UserRole + 2;
static const int SearchPath = Dtk::UserRole + 3;
static const int DummyStatusRole = Dtk::UserRole + 4;

const QSize ListViewItemCommonSize(168,58);
const QSize ListViewItemCommonCompactModeSize(168,46);

NetworkModuleWidget::NetworkModuleWidget(QWidget *parent)
    : QWidget(parent)
    , m_lvnmPages(new CompactListView(ListViewItemCommonSize, ListViewItemCommonCompactModeSize, this))
    , m_modelPages(new QStandardItemModel(this))
    , m_nmConnectionEditorProcess(nullptr)
    , m_frameProxy(nullptr)
    , m_isFirstEnter(true)
    , m_switchIndex(true)
{
    setObjectName("Network");
    m_lvnmPages->setAccessibleName("List_networkmenulist");
    m_lvnmPages->setFrameShape(QFrame::NoFrame);
    m_lvnmPages->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_lvnmPages->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_lvnmPages->setModel(m_modelPages);
    m_lvnmPages->setViewportMargins(ScrollAreaMargins);
    m_lvnmPages->setIconSize(ListViweIconSize);

    m_centralLayout = new QVBoxLayout();
    setMinimumWidth(250);
    m_centralLayout->setMargin(0);
    setLayout(m_centralLayout);

    auto *pppIt = new DStandardItem(tr("DSL"));
    pppIt->setData(QVariant::fromValue(PageType::DSLPage), SectionRole);
    pppIt->setIcon(QIcon::fromTheme("dcc_dsl"));
    m_modelPages->appendRow(pppIt);
    GSettingWatcher::instance()->bind("networkDsl", m_lvnmPages, pppIt);

    auto *vpnit = new DStandardItem(tr("VPN"));
    vpnit->setData(QVariant::fromValue(PageType::VPNPage), SectionRole);
    vpnit->setIcon(QIcon::fromTheme("dcc_vpn"));
    m_modelPages->appendRow(vpnit);
    GSettingWatcher::instance()->bind("networkVpn", m_lvnmPages, vpnit);

    auto *prxyit = new DStandardItem(tr("System Proxy"));
    prxyit->setData(QVariant::fromValue(PageType::SysProxyPage), SectionRole);
    prxyit->setIcon(QIcon::fromTheme("dcc_system_agent"));
    m_modelPages->appendRow(prxyit);
    GSettingWatcher::instance()->bind("systemProxy", m_lvnmPages, prxyit);

    auto *aprxit = new DStandardItem(tr("Application Proxy"));
    aprxit->setData(QVariant::fromValue(PageType::AppProxyPage), SectionRole);
    aprxit->setIcon(QIcon::fromTheme("dcc_app_proxy"));
    m_modelPages->appendRow(aprxit);
    GSettingWatcher::instance()->bind("applicationProxy", m_lvnmPages, aprxit);

    //~ contents_path /network/Airplane
    auto *airplaneMode = new DStandardItem(tr("Airplane Mode"));
    airplaneMode->setData(QVariant::fromValue(PageType::AirplaneModepage), SectionRole);
    airplaneMode->setIcon(QIcon::fromTheme("dcc_airplane_mode"));
    m_modelPages->appendRow(airplaneMode);

    if (IsServerSystem)
        handleNMEditor();

    m_lvnmPages->viewport()->installEventFilter(this);
    connect(m_lvnmPages, &DListView::activated, m_lvnmPages, &DListView::clicked);
    connect(m_lvnmPages, &CompactListView::clicked, this, [ this ] (const QModelIndex &index) {
        selectListIndex(index);
    });

    NetworkController *pNetworkController = NetworkController::instance();
    connect(pNetworkController, &NetworkController::activeConnectionChange, this, &NetworkModuleWidget::onDeviceStatusChanged);
    connect(pNetworkController, &NetworkController::deviceRemoved, this, &NetworkModuleWidget::onDeviceChanged);
    connect(pNetworkController, &NetworkController::deviceAdded, this, [ = ] () {
        onDeviceChanged();
        if (m_isFirstEnter) {
            showDefaultWidget();
            m_isFirstEnter = false;
        }
    });

    ProxyController *proxyController = pNetworkController->proxyController();
    connect(proxyController, &ProxyController::proxyMethodChanged, this, &NetworkModuleWidget::onProxyMethodChanged);

    onDeviceChanged();

    auto *infoIt = new DStandardItem(tr("Network Details"));
    infoIt->setData(QVariant::fromValue(PageType::NetworkInfoPage), SectionRole);
    infoIt->setIcon(QIcon::fromTheme("dcc_network"));
    m_modelPages->appendRow(infoIt);
    GSettingWatcher::instance()->bind("networkDetails", m_lvnmPages, infoIt);

    m_centralLayout->addWidget(m_lvnmPages);
    QTimer::singleShot(0, this, [ proxyController, this ] {
        proxyController->querySysProxyData();
        onProxyMethodChanged(proxyController->proxyMethod());
    });

    QDBusConnection::systemBus().connect("org.freedesktop.login1", "/org/freedesktop/login1", "org.freedesktop.login1.Manager", "PrepareForSleep", this, SLOT(onPrepareForSleep(bool)));
    if (!NetworkController::instance()->devices().empty())
        m_isFirstEnter = false;
}

NetworkModuleWidget::~NetworkModuleWidget()
{
    if (m_nmConnectionEditorProcess) {
        m_nmConnectionEditorProcess->close();
        m_nmConnectionEditorProcess->deleteLater();
        m_nmConnectionEditorProcess = nullptr;
    }
}

void NetworkModuleWidget::initIpConflictInfo(const QList<NetworkDeviceBase *> &devices)
{
    for (NetworkDeviceBase *device : devices) {
        connect(device, &NetworkDeviceBase::deviceStatusChanged, this, [](const DeviceStatus &deviceStatus) {
            if (deviceStatus == DeviceStatus::IpConflict) {
                Notifications notifications("org.freedesktop.Notifications", "/org/freedesktop/Notifications", QDBusConnection::sessionBus());
                notifications.Notify("dde-control-center", static_cast<uint>(QDateTime::currentMSecsSinceEpoch()), "preferences-system", tr("Network"), tr("IP conflict"), QStringList(), QVariantMap(), 3000);
            }
        });
    }
}

void NetworkModuleWidget::selectListIndex(const QModelIndex &idx)
{
    if (!m_switchIndex)
        return;

    const QString searchPath = idx.data(SearchPath).toString();
    m_modelPages->itemFromIndex(idx)->setData("", SearchPath);

    auto type = idx.data(SectionRole).value<PageType>();
    m_lvnmPages->setCurrentIndex(idx);
    switch (type) {
    case PageType::DSLPage:
        Q_EMIT requestShowPppPage(searchPath);
        break;
    case PageType::VPNPage:
        Q_EMIT requestShowVpnPage(searchPath);
        break;
    case PageType::SysProxyPage:
        Q_EMIT requestShowProxyPage();
        break;
    case PageType::AppProxyPage:
        Q_EMIT requestShowChainsPage();
        break;
    case PageType::HotspotPage:
        Q_EMIT requestHotspotPage(searchPath);
        break;
    case PageType::NetworkInfoPage:
        Q_EMIT requestShowInfomation();
        break;
    case PageType::WiredPage:
    case PageType::WirelessPage:
        if (!m_currentSearchPath.isEmpty()) {
            Q_EMIT requestShowDeviceDetail(idx.data(DeviceRole).value<NetworkDeviceBase *>(), m_currentSearchPath);
        } else {
            Q_EMIT requestShowDeviceDetail(idx.data(DeviceRole).value<NetworkDeviceBase *>(), searchPath);
        }
        m_currentSearchPath = QString{};
        break;
    case PageType::AirplaneModepage:
        Q_EMIT requestShowAirplanePage();
        break;
    default:
        break;
    }

    m_lvnmPages->resetStatus(idx);
}

void NetworkModuleWidget::onProxyMethodChanged(const ProxyMethod &method)
{
    if (method == ProxyMethod::Init) return;
    QPointer<DViewItemAction> action(new DViewItemAction(Qt::AlignmentFlag::AlignRight | Qt::AlignmentFlag::AlignVCenter));
    if (action.isNull()) return;
    //遍历获取系统代理项,设置状态
    for (int i = 0; i < m_modelPages->rowCount(); i++) {
    auto *item = dynamic_cast<DStandardItem *>(m_modelPages->item(i));
    if (!item)
        continue;

    if (item->data(SectionRole).value<PageType>() == PageType::SysProxyPage) {
        item->setActionList(Qt::Edge::RightEdge, {action});
        if (method == ProxyMethod::None)
            action->setText(tr("Disabled"));
        else if (method == ProxyMethod::Manual)
            action->setText(tr("Manual"));
        else if (method == ProxyMethod::Auto)
            action->setText(tr("Auto"));
        else
            action->setText(tr("Disabled"));

        break;
    }
    }
}

void NetworkModuleWidget::onPrepareForSleep(bool state)
{
    // 休眠或者待机state:true(待机或休眠) false:唤醒
    if (!m_modelPages || !m_lvnmPages)
        return;

    if (state) {
        // 当待机的时候,记录当前选中项
        m_selectItemName = m_lvnmPages->currentIndex().data(Qt::DisplayRole).toString();
        return;
    }

    for (int i = 0; i < m_modelPages->rowCount(); i++) {
        const QModelIndex currentIndex = m_modelPages->index(i, 0);
        if (currentIndex.data(Qt::DisplayRole).toString() == m_selectItemName) {
            m_lvnmPages->setCurrentIndex(currentIndex);
            break;
        }
    }
}

bool NetworkModuleWidget::handleNMEditor()
{
    auto *process = new QProcess(this);
    auto *nmConnEditBtn = new QPushButton(tr("Configure by Network Manager"));
    m_centralLayout->addWidget(nmConnEditBtn);
    nmConnEditBtn->hide();
    process->start("which nm-connection-editor");

    connect(process, static_cast<void (QProcess::*)(int)>(&QProcess::finished), this, [ = ] {
        QString networkManageOutput = process->readAll();
        if (!networkManageOutput.isEmpty()) {
            nmConnEditBtn->show();
            connect(nmConnEditBtn, &QPushButton::clicked, this, [ = ] {
                if (!m_nmConnectionEditorProcess) {
                    m_nmConnectionEditorProcess = new QProcess(this);
                }
                m_nmConnectionEditorProcess->start("nm-connection-editor");
            });
        }
        process->deleteLater();
    });

    return true;
}

void NetworkModuleWidget::showDefaultWidget()
{
    for (int i = 0; i < m_modelPages->rowCount(); i++) {
        if (!m_lvnmPages->isRowHidden(i)) {
            setCurrentIndex(i);
            break;
        }
    }
}

void NetworkModuleWidget::setCurrentIndex(const int settingIndex)
{
    // 设置网络列表当前索引
    QModelIndex index = m_modelPages->index(settingIndex, 0);
    m_lvnmPages->setCurrentIndex(index);
    selectListIndex(index);
}

void NetworkModuleWidget::setIndexFromPath(const QString &path)
{
    for (int i = 0; i < m_modelPages->rowCount(); ++i) {
        QString indexPath =
            m_modelPages->item(i)->data(DeviceRole).value<NetworkDeviceBase *>()->path();
        if (indexPath == path) {
            setCurrentIndex(i);
            return;
        }
    }
}

void NetworkModuleWidget::initSetting(const int settingIndex, const QString &searchPath)
{
    QModelIndex currentIndex = m_modelPages->index(settingIndex, 0);
    if (!searchPath.isEmpty())
        m_modelPages->itemFromIndex(currentIndex)->setData(searchPath, SearchPath);

    if (m_lvnmPages->currentIndex() != currentIndex) {
        m_lvnmPages->setCurrentIndex(currentIndex);
        m_lvnmPages->clicked(m_modelPages->index(settingIndex, 0));
    } else {
        selectListIndex(currentIndex);
    }
}

static PageType getPageTypeFromModelName(const QString &modelName)
{
    if (modelName == "networkWired")
        return PageType::WiredPage;

    if (modelName == "networkWireless")
        return PageType::WirelessPage;

    if (modelName == "personalHotspot")
        return PageType::HotspotPage;

    if (modelName =="applicationProxy")
        return PageType::AppProxyPage;

    if (modelName == "networkDetails")
        return PageType::NetworkInfoPage;

    if (modelName == "networkDsl")
        return PageType::DSLPage;

    if (modelName == "systemProxy")
        return PageType::SysProxyPage;

    if (modelName == "networkVpn")
        return PageType::VPNPage;

    if (modelName == "networkAirplane")
        return PageType::AirplaneModepage;

    return PageType::NonePage;
}

void NetworkModuleWidget::setModelVisible(const QString &modelName, const bool &visible)
{
    PageType type = getPageTypeFromModelName(modelName);
    for (int i = 0; i < m_modelPages->rowCount(); i++) {
        if (m_modelPages->item(i)->data(SectionRole).value<PageType>() == type) {
            m_lvnmPages->setRowHidden(i, !visible);
        }
    }
}

int NetworkModuleWidget::gotoSetting(const QString &path)
{
    PageType type = PageType::NonePage;
    if (path == QStringLiteral("Network Details")) {
        type = PageType::NetworkInfoPage;
    } else if (path == QStringLiteral("Application Proxy")) {
        type = PageType::AppProxyPage;
    } else if (path == QStringLiteral("System Proxy")) {
        type = PageType::SysProxyPage;
    } else if (path == QStringLiteral("VPN") || path == QStringLiteral("Create VPN") || path == QStringLiteral("Import VPN")) {
        type = PageType::VPNPage;
    } else if (path == QStringLiteral("DSL") || path.contains(QStringLiteral("PPPoE"))) {
        type = PageType::DSLPage;
    } else if (path.contains("WirelessPage") || path.contains("Wireless Network")
               || path.contains("Connect to hidden network")) {
        // 历史原因，WirelessPage可能也在使用中
        type = PageType::WirelessPage;
    } else if (path.contains("Wired Network") || path.contains("addWiredConnection")) {
        type = PageType::WiredPage;
    } else if (path == QStringLiteral("Personal Hotspot")) {
        type = PageType::HotspotPage;
    } else if (path == QStringLiteral("Airplane Mode")) {
        type = PageType::AirplaneModepage;
    }
    int index = -1;
    for (int i = 0; i < m_modelPages->rowCount(); ++i) {
        if (m_modelPages->item(i)->data(SectionRole).value<PageType>() == type) {
            index = i;
            break;
        }
    }

    return index;
}

void NetworkModuleWidget::onDeviceStatusChanged()
{
    for (int i = 0; i < m_modelPages->rowCount(); i++) {
        auto *item = dynamic_cast<DStandardItem *>(m_modelPages->item(i));
        auto pageType = item->data(SectionRole).value<PageType>();
        if (pageType != PageType::WiredPage && pageType != PageType::WirelessPage)
            continue;

        auto *device = item->data(DeviceRole).value<NetworkDeviceBase *>();
        if (!device)
            continue;

        QPointer<DViewItemAction> dummyStatus = item->data(DummyStatusRole).value<QPointer<DViewItemAction>>();
        QString txt = device->isEnabled() ? device->property("statusName").toString() : tr("Disabled");
        dummyStatus->setText(txt);
    }
}

static bool wiredExist(const QList<NetworkDeviceBase *> &devices)
{
    return (std::any_of(devices.cbegin(), devices.cend(), [ devices ](const NetworkDeviceBase *device) -> bool {
        return (device->deviceType() == DeviceType::Wired);
    }));
}

void NetworkModuleWidget::onDeviceChanged()
{
    QList<NetworkDeviceBase *> devices = NetworkController::instance()->devices();

    // 查询是否存在有线网络，来决定是否显示DSL模块，只要存在有线网络，就显示DSL，否则，不显示DSL
    bool hasWired = wiredExist(devices);
    setModelVisible("networkDsl", hasWired);

    // 在新增项或删除项的过程中会触发QItemSelectionModel::currentChanged的信号，因此，先将此处的信号进行阻塞
    m_lvnmPages->selectionModel()->blockSignals(true);

    QModelIndex currentIndex = m_lvnmPages->currentIndex();
    auto pageType = currentIndex.data(SectionRole).value<PageType>();
    auto *currentDevice = currentIndex.data(DeviceRole).value<NetworkDeviceBase *>();
    // 如果当前选择的是隐藏项，则让其默认为NonePage，此时选中的就是第一个页面
    if (m_lvnmPages->isRowHidden(currentIndex.row()))
        pageType = PageType::NonePage;

    for (int i = m_modelPages->rowCount() - 1; i >= 0; i--) {
        QStandardItem *item = m_modelPages->item(i);
        auto itemPageType = item->data(SectionRole).value<PageType>();
        if (itemPageType != PageType::WiredPage && itemPageType != PageType::WirelessPage &&
            itemPageType != PageType::HotspotPage)
            continue;

        m_modelPages->removeRow(i);
    }

    bool supportHotspot = false;
    for (NetworkDeviceBase *device : devices) {
        if (device->supportHotspot()) {
            supportHotspot = true;
            break;
        }
    }

    m_switchIndex = true;
    int newRowIndex = -1;
    for (int i = 0; i < devices.size(); i++) {
        NetworkDeviceBase *device = devices[i];
        auto *deviceItem = new DStandardItem(device->deviceName());
        if (device->deviceType() == DeviceType::Wireless) {
            deviceItem->setData(QVariant::fromValue(PageType::WirelessPage), SectionRole);
            deviceItem->setIcon(QIcon::fromTheme("dcc_wifi"));
        } else if (device->deviceType() == DeviceType::Wired){
            deviceItem->setData(QVariant::fromValue(PageType::WiredPage), SectionRole);
            deviceItem->setIcon(QIcon::fromTheme("dcc_ethernet"));
        } else {
            continue;
        }

        deviceItem->setData(QVariant::fromValue(device), DeviceRole);

        QPointer<DViewItemAction> dummyStatus(new DViewItemAction(Qt::AlignmentFlag::AlignRight | Qt::AlignmentFlag::AlignVCenter));
        deviceItem->setActionList(Qt::Edge::RightEdge, { dummyStatus });
        deviceItem->setData(QVariant::fromValue(dummyStatus), DummyStatusRole);

        if (!dummyStatus.isNull()) {
            if (device->isEnabled())
            dummyStatus->setText(device->property("statusName").toString());
            else
            dummyStatus->setText(tr("Disabled"));

            m_lvnmPages->update();
        }

        auto setDeviceStatus = [ this, device, dummyStatus ] {
            if (!dummyStatus.isNull()) {
                QString txt = device->isEnabled() ? device->property("statusName").toString() : tr("Disabled");
                dummyStatus->setText(txt);
            }
            this->m_lvnmPages->update();
        };

        connect(device, &NetworkDeviceBase::enableChanged, this, setDeviceStatus);
        connect(device, &NetworkDeviceBase::deviceStatusChanged, this, setDeviceStatus);
        if (device->deviceType() == DeviceType::Wireless) {
            auto *wirelessDev = qobject_cast<WirelessDevice *>(device);
            if (wirelessDev)
                connect(wirelessDev, &WirelessDevice::hotspotEnableChanged, this, setDeviceStatus);
        }

        m_modelPages->insertRow(i, deviceItem);

        if (pageType == PageType::WiredPage || pageType == PageType::WirelessPage) {
            // 如果是有线网络或者无线网络，则根据之前的设备和当前的设备是否相同来获取索引
            // 此时调用setCurrentIndex无需再次创建三级菜单窗口，因此，将m_switchIndex赋值为false
            if (currentDevice == device) {
                newRowIndex = i;
                m_switchIndex = false;
            }
        }
    }
    if (supportHotspot) {
        auto *hotspotItem = new DStandardItem(tr("Personal Hotspot"));
        hotspotItem->setData(QVariant::fromValue(PageType::HotspotPage), SectionRole);
        hotspotItem->setIcon(QIcon::fromTheme("dcc_hotspot"));
        int hotspotRow = m_modelPages->rowCount() - 1;
        m_modelPages->insertRow(hotspotRow, hotspotItem);
        GSettingWatcher::instance()->bind("personalHotspot", m_lvnmPages, hotspotItem);
    }
    // 获取之前的索引就和当前的索引对比
    if (newRowIndex < 0) {
        for (int i = 0; i < m_modelPages->rowCount(); i++) {
            QStandardItem *currentRowItem = m_modelPages->item(i);
            auto currentPageType = currentRowItem->data(SectionRole).value<PageType>();
            if (currentPageType == PageType::WiredPage || currentPageType == PageType::WirelessPage)
                continue;

            if (pageType == currentPageType) {
                newRowIndex = i;
                m_switchIndex = false;
            }
        }
    }

    if (newRowIndex < 0) {
        for (int i = 0; i < m_modelPages->rowCount(); i++) {
            if (!m_lvnmPages->isRowHidden(i)) {
                newRowIndex = i;
                break;
            }
        }
    }

    m_lvnmPages->selectionModel()->blockSignals(false);

    setCurrentIndex(newRowIndex);
    m_switchIndex = true;

    if (!m_selectItemName.isNull())
        onPrepareForSleep(false);
}

void NetworkModuleWidget::setCurrentSearchPath(NetworkDeviceBase *dev,const QStringList &searchPath) {
    for (int i = 0; i < m_modelPages->rowCount(); ++i) {
        auto currentDev = m_modelPages->item(i)->data(DeviceRole).value<NetworkDeviceBase *>();
        if (currentDev){
            QString indexPath = currentDev->path();
            if (indexPath == searchPath.at(0)) {
                if (i == m_lvnmPages->currentIndex().row()){
                  // 当前index就是需要的index，直接显示即可
                  emit requestShowDeviceDetail(dev, searchPath.at(1));
                  return;
                }
            }
        }
    }
    // 当前index不是需要的index，需要先设置searchPath，再通过currentChanged信号触发显示
    m_currentSearchPath = searchPath.at(1);
    setIndexFromPath(searchPath.at(0));
}

bool NetworkModuleWidget::eventFilter(QObject *w, QEvent *e)
{
    if (e->type() == QEvent::MouseButtonRelease && w == m_lvnmPages->viewport()) {
        if (m_lvnmPages->currentIndex().row() != -1) {
            selectListIndex(m_lvnmPages->currentIndex());
        }
    }
    return QWidget::eventFilter(w, e);
}
