// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "vpnpage.h"
#include "connectionvpneditpage.h"
#include "connectionpageitem.h"
#include "widgets/contentwidget.h"
#include "widgets/switchwidget.h"
#include "widgets/titlelabel.h"
#include "widgets/translucentframe.h"
#include "widgets/loadingnextpagewidget.h"
#include "window/gsettingwatcher.h"

#include <DFloatingButton>
#include <DHiDPIHelper>
#include <ddialog.h>
#include <networkmanagerqt/settings.h>
#include <DSpinner>

#include <QList>
#include <QMap>
#include <QVBoxLayout>
#include <QPushButton>
#include <QJsonObject>
#include <QFileDialog>
#include <QMessageBox>
#include <QProcess>
#include <QRegularExpression>
#include <QStandardItemModel>

#include <networkcontroller.h>
#include <vpncontroller.h>

DWIDGET_USE_NAMESPACE

using namespace dcc;
using namespace dcc::widgets;
using namespace dde::network;
using namespace NetworkManager;

#define ServiceTypeL2TP         "org.freedesktop.NetworkManager.l2tp"
#define ServiceTypePPTP         "org.freedesktop.NetworkManager.pptp"
#define ServiceTypeVPNC         "org.freedesktop.NetworkManager.vpnc"
#define ServiceTypeOpenVPN      "org.freedesktop.NetworkManager.openvpn"
#define ServiceTypeStrongSwan   "org.freedesktop.NetworkManager.strongswan"
#define ServiceTypeOpenConnect  "org.freedesktop.NetworkManager.openconnect"
#define ServiceTypeSSTP         "org.freedesktop.NetworkManager.sstp"

QString vpnConfigType(const QString &path)
{
    QFile f(path);
    if (!f.open(QIODevice::ReadOnly))
        return QString();

    const QString content = f.readAll();
    f.close();

    if (content.contains("openconnect"))
        return "openconnect";
    if (content.contains("l2tp"))
        return "l2tp";
    if (content.startsWith("[main]"))
        return "vpnc";

    return "openvpn";
}

VpnPage::VpnPage(QWidget *parent)
    : QWidget(parent)
    , m_lvProfiles(new DListView(this))
    , m_modelProfiles(new QStandardItemModel(this))
    , m_importFile(nullptr)
{
    m_lvProfiles->setAccessibleName("List_vpnList");
    m_lvProfiles->setModel(m_modelProfiles);
    m_lvProfiles->setEditTriggers(QAbstractItemView::NoEditTriggers);
    m_lvProfiles->setBackgroundType(DStyledItemDelegate::BackgroundType::ClipCornerBackground);
    m_lvProfiles->setSelectionMode(QAbstractItemView::NoSelection);

    auto *widget = new QWidget(this);
    auto *scrollLayout = new QVBoxLayout(widget);

    auto *lblTitle = new QLabel(tr("VPN Status"), widget);
    DFontSizeManager::instance()->bind(lblTitle, DFontSizeManager::T5, QFont::DemiBold);
    m_vpnSwitch = new SwitchWidget(widget, lblTitle);
    m_vpnSwitch->switchButton()->setAccessibleName(lblTitle->text());
    // 因为swtichbutton内部距离右间距为4,所以这里设置6就可以保证间距为10
    m_vpnSwitch->getMainLayout()->setContentsMargins(10, 0, 6, 0);

    scrollLayout->addWidget(m_vpnSwitch, 0, Qt::AlignTop);
    // 控制lisview字体间距
    QMargins itemMargins(m_lvProfiles->itemMargins());
    itemMargins.setLeft(2);
    m_lvProfiles->setItemMargins(itemMargins);
    scrollLayout->addWidget(m_lvProfiles);
    scrollLayout->setSpacing(10);
    scrollLayout->setContentsMargins(QMargins(10, 0, 10, 0));  // 设置左右间距为10

    widget->setLayout(scrollLayout);

    auto *contentWidget = new ContentWidget(this);
    contentWidget->setAccessibleName("VpnPage_ContentWidget");
    contentWidget->layout()->setMargin(0);
    contentWidget->setContent(widget);

    auto *mainLayout = new QVBoxLayout(this);
    mainLayout->setMargin(0);
    mainLayout->addWidget(contentWidget);

    auto *buttonsLayout = new QHBoxLayout(this);
    buttonsLayout->setSpacing(30);
    buttonsLayout->addStretch();

    auto *createVpnBtn = new DFloatingButton(DStyle::StandardPixmap::SP_IncreaseElement, this);
    createVpnBtn->setMinimumSize(QSize(47, 47));
    createVpnBtn->setToolTip(tr("Create VPN"));
    createVpnBtn->setAccessibleName(tr("Create VPN"));
    GSettingWatcher::instance()->bind("createVpn", createVpnBtn);
    buttonsLayout->addWidget(createVpnBtn);

    auto *importVpnBtn = new DFloatingButton(this);
    importVpnBtn->setMinimumSize(QSize(47, 47));
    importVpnBtn->setToolTip(tr("Import VPN"));
    importVpnBtn->setAccessibleName(tr("Import VPN"));
    importVpnBtn->setIcon(QIcon::fromTheme("dcc_importvpn"));
    GSettingWatcher::instance()->bind("importVpn", importVpnBtn);
    buttonsLayout->addWidget(importVpnBtn);
    buttonsLayout->addStretch();
    mainLayout->addLayout(buttonsLayout);

    setContentsMargins(0, 10, 0, 7);                        //设置上间距为10，设计要求“+”按钮与底部间隔10px，而组件自带3像素间隔，所以此处设7px
    setLayout(mainLayout);

    VPNController *vpnController = NetworkController::instance()->vpnController();

    connect(m_lvProfiles, &DListView::clicked, this, [=](const QModelIndex &index) {
        QString uuid = index.data(UuidRole).toString();
        ConnectionPageItem *pageItem = dynamic_cast<ConnectionPageItem *>(m_modelProfiles->item(index.row()));
        VPNItem *vpn = pageItem ? static_cast<VPNItem *>(pageItem->itemData()) : nullptr;
        if (settingIsValid(vpn)) {
            vpnController->connectItem(uuid);
        } else {
            // 如果配置不合法，就让其弹出配置的页面进行配置
            showEditPage(vpn, true);
        }
    });
    connect(m_vpnSwitch, &SwitchWidget::checkedChanged, this, [ = ](const bool checked) {
        vpnController->setEnabled(checked);
    });

    connect(createVpnBtn, &QPushButton::clicked, this, &VpnPage::createVPN);
    connect(importVpnBtn, &QPushButton::clicked, this, &VpnPage::importVPN);

    connect(vpnController, &VPNController::enableChanged, m_vpnSwitch, &SwitchWidget::setChecked);
    connect(vpnController, &VPNController::activeConnectionChanged, this, &VpnPage::onActiveConnsInfoChanged);
    connect(vpnController, &VPNController::enableChanged, this, &VpnPage::onActiveConnsInfoChanged);
    connect(vpnController, &VPNController::itemChanged, this, &VpnPage::updateVpnItems);
    connect(vpnController, &VPNController::itemAdded, this, [ = ] {
        QList<VPNItem *> items = vpnController->items();
        refreshVpnList(items);
        if (!m_newConnectionPath.isEmpty()) {
            for (VPNItem *item : items) {
                if (item->connection()->path() == m_newConnectionPath) {
                    vpnController->connectItem(item);
                    m_newConnectionPath.clear();
                    break;
                }
            }
        }
    });
    connect(vpnController, &VPNController::itemRemoved, this, [ = ] {
        refreshVpnList(vpnController->items());
    });

    m_vpnSwitch->setChecked(vpnController->enabled());

    refreshVpnList(vpnController->items());
}

VpnPage::~VpnPage()
{
    GSettingWatcher::instance()->erase("createVpn");
    GSettingWatcher::instance()->erase("importVpn");

    if (!m_editPage.isNull())
        m_editPage->deleteLater();
}

bool VpnPage::settingIsValid(VPNItem *item) const
{
    if (!item || (item && !m_editPage.isNull() && m_editPage->connectionUuid() != item->connection()->uuid()))
        return false;

    NetworkManager::Connection::Ptr connection = NetworkManager::findConnectionByUuid(item->connection()->uuid());
    if (connection.isNull())
        return false;

    QSharedPointer<VPNParametersChecker> checker(VPNParametersChecker::createVpnChecker(connection));
    return checker->isValid();
}

void VpnPage::refreshVpnList(const QList<VPNItem *>& vpns)
{
    m_modelProfiles->clear();

    for (VPNItem *vpn : vpns) {
        const QString uuid = vpn->connection()->uuid();

        auto *vpnItem = new ConnectionPageItem(this, m_lvProfiles, vpn->connection());
        vpnItem->setText(vpn->connection()->id());
        vpnItem->setItemData(vpn);
        vpnItem->setData(uuid, UuidRole);

        connect(vpnItem, &ConnectionPageItem::detailClick, this, [ = ] {
            showEditPage(vpn);
        });

        m_modelProfiles->appendRow(vpnItem);
    }

    //onActiveConnsInfoChanged(m_model->activeConnInfos());
    m_vpnSwitch->setVisible(m_modelProfiles->rowCount() > 0);

    // 延迟刷新，是为了显示正常
    QTimer::singleShot(1, this, [ = ] {
        onActiveConnsInfoChanged();
    });
}

void VpnPage::updateVpnItems(const QList<VPNItem *> &vpns)
{
    for (int i = 0; i < m_modelProfiles->rowCount(); i++) {
        auto *pageItem = dynamic_cast<ConnectionPageItem *>(m_modelProfiles->item(i));
        auto *vpn = static_cast<VPNItem *>(pageItem->itemData());
        if (vpns.contains(vpn))
            pageItem->setText(vpn->connection()->id());
    }
}

void VpnPage::onActiveConnsInfoChanged()
{
    bool vpnEnabled = NetworkController::instance()->vpnController()->enabled();
    for (int i = 0; i < m_modelProfiles->rowCount(); ++i) {
        auto *item = dynamic_cast<ConnectionPageItem *>(m_modelProfiles->item(i));
        auto *vpnItem = static_cast<VPNItem *>(item->itemData());
        if (!vpnItem)
            continue;

        item->setConnectionStatus(vpnEnabled ? vpnItem->status() : ConnectionStatus::Deactivated);
    }
}

void VpnPage::changeVpnId()
{
    NetworkManager::Connection::List connList = listConnections();
    QString importName = "";
    for (const auto &conn : connList) {
        if (conn->settings()->connectionType() == ConnectionSettings::Vpn) {
            if (m_editingConnUuid == conn->uuid()) {
                importName = conn->name();
                break;
            }
        }
    }
    if (importName.isEmpty()) {
        QTimer::singleShot(10, this, &VpnPage::changeVpnId);
        return;
    }

    QString changeName = "";
    bool hasSameName = false;
    for (const auto &conn : connList) {
        const QString vpnName = conn->name();
        const QString vpnUuid = conn->uuid();
        if ((vpnName == importName) && (vpnUuid != m_editingConnUuid)) {
            changeName = importName + "(1)";
            hasSameName = true;
            break;
        }
    }
    if (!hasSameName) {
        return;
    }

    for (int index = 1; ; index++) {
        hasSameName = false;
        for (const auto &conn : connList) {
            QString vpnName = conn->name();
            if (vpnName == changeName) {
                changeName = importName + "(%1)";
                changeName = changeName.arg(index);
                hasSameName = true;
                break;
            }
        }
        if (!hasSameName) {
            break;
        }
    }

    NetworkManager::Connection::Ptr uuidConn = findConnectionByUuid(m_editingConnUuid);
    if (uuidConn) {
        ConnectionSettings::Ptr connSettings = uuidConn->settings();
        connSettings->setId(changeName);
        // update function saves the settings on the hard disk
        QDBusPendingReply<> reply = uuidConn->update(connSettings->toMap());
        reply.waitForFinished();
        if (reply.isError()) {
            qDebug() << "error occurred while updating the connection" << reply.error();
            return;
        }
        qDebug() << "find Connection By Uuid is success";
        return;
    }
}

void VpnPage::showEditPage(VPNItem *vpn, bool needCheck)
{
    m_editPage = new ConnectionVpnEditPage(vpn ? vpn->connection()->uuid() : "");
    m_editPage->setNeedCheck(needCheck);
    m_editPage->initSettingsWidget();
    m_editPage->setLeftButtonEnable(true);

    connect(m_editPage, &ConnectionVpnEditPage::requestNextPage, this, &VpnPage::requestNextPage);
    connect(m_editPage, &ConnectionVpnEditPage::requestFrameAutoHide, this, &VpnPage::requestFrameKeepAutoHide);
    connect(m_editPage, &ConnectionVpnEditPage::disconnect, this, [ = ] {
        VPNController *vpnController = NetworkController::instance()->vpnController();
        vpnController->disconnectItem();
    });
    connect(m_editPage, &ConnectionVpnEditPage::activateVpnConnection, this, [ vpn ] {
        VPNController *vpnController = NetworkController::instance()->vpnController();
        vpnController->connectItem(vpn);
    });

    Q_EMIT requestNextPage(m_editPage);
}

void VpnPage::importVPN()
{
    Q_EMIT requestFrameKeepAutoHide(false);

    if (!m_importFile) {
        m_importFile = new QFileDialog(this);
        m_importFile->setAccessibleName("VpnPage_importFile");
        m_importFile->setModal(true);
        m_importFile->setNameFilter("*.conf");
        m_importFile->setAcceptMode(QFileDialog::AcceptOpen);
        QStringList directory = QStandardPaths::standardLocations(QStandardPaths::HomeLocation);
        if (!directory.isEmpty()) {
            m_importFile->setDirectory(directory.first());
        }
    }

    m_importFile->exec();
    Q_EMIT requestFrameKeepAutoHide(true);

    if (m_importFile->result() != QFileDialog::Accepted)
        return;

    QString file = m_importFile->selectedFiles().first();
    if (file.isEmpty())
        return;

    const auto args = QStringList { "connection", "import", "type", vpnConfigType(file), "file", file };

    QProcess p;
    p.start("nmcli", args);
    p.waitForFinished();
    const auto stat = p.exitCode();
    const QString output = p.readAllStandardOutput();
    QString error = p.readAllStandardError();

    qDebug() << stat << ",output:" << output << ",err:" << error;

    if (stat) {
        const auto ratio = devicePixelRatioF();
        QPixmap icon = QIcon::fromTheme("dialog-error").pixmap(QSize(48, 48) * ratio);
        icon.setDevicePixelRatio(ratio);

        DDialog dialog(this);
        dialog.setTitle(tr("Import Error"));
        dialog.setMessage(tr("File error"));
        dialog.addButton(tr("OK"));
        dialog.setIcon(icon);
        dialog.exec();
        return;
    }

    const QRegularExpression regexp(R"(\(\w{8}(-\w{4}){3}-\w{12}\))");
    const auto match = regexp.match(output);

    if (match.hasMatch()) {
        m_editingConnUuid = match.captured();
        m_editingConnUuid.replace("(", "");
        m_editingConnUuid.replace(")", "");
        qDebug() << "editing connection Uuid";
        QTimer::singleShot(10, this, &VpnPage::changeVpnId);
    }
}

void VpnPage::createVPN()
{
    m_editPage = new ConnectionVpnEditPage("", this);
    m_editPage->initSettingsWidgetByType(ConnectionVpnEditPage::VpnType::UNSET);
    connect(m_editPage, &ConnectionVpnEditPage::requestNextPage, this, &VpnPage::requestNextPage);
    connect(m_editPage, &ConnectionVpnEditPage::requestFrameAutoHide, this, &VpnPage::requestFrameKeepAutoHide);
    connect(m_editPage, &ConnectionVpnEditPage::activateVpnConnection, this, [ = ] (const QString &path, const QString &devicePath) {
        Q_UNUSED(devicePath);

        m_newConnectionPath.clear();
        bool findConnection = false;
        VPNController *vpnController = NetworkController::instance()->vpnController();
        QList<VPNItem *> items = vpnController->items();
        for (VPNItem *item : items) {
            if (item->connection()->path() == path) {
                vpnController->connectItem(item);
                findConnection = true;
                break;
            }
        }
        if (!findConnection)
            m_newConnectionPath = path;
    });
    Q_EMIT requestNextPage(m_editPage);

    //only create New Connection can set "Cancel","Save" button
    //fix: https://pms.uniontech.com/zentao/bug-view-61563.html
    m_editPage->setButtonTupleEnable(true);
}

void VpnPage::jumpPath(const QString &searchPath)
{
    if (searchPath == "Create VPN")
        createVPN();

    if (searchPath == "Import VPN")
        importVPN();
}

// 参数校验
VPNParametersChecker::VPNParametersChecker(const VpnSetting::Ptr &vpnSetting)
    : m_vpnSetting(vpnSetting)
{
}

VPNParametersChecker::~VPNParametersChecker()
{
}

bool VPNParametersChecker::isValid() const
{
    return true;
}

VPNParametersChecker *VPNParametersChecker::createVpnChecker(const NetworkManager::Connection::Ptr &connection)
{
    if (connection->settings()->connectionType() != NetworkManager::ConnectionSettings::ConnectionType::Vpn)
        return new DefaultChecker;

    if (connection->settings()->id().isEmpty() || connection->settings()->name().isEmpty())
        return new DefaultChecker;

    QDBusPendingReply<NMVariantMapMap> reply = connection->secrets(connection->settings()->setting(NetworkManager::Setting::SettingType::Vpn)->name());
    reply.waitForFinished();
    NetworkManager::VpnSetting::Ptr setting = connection->settings()->setting(NetworkManager::Setting::SettingType::Vpn).staticCast<NetworkManager::VpnSetting>();
    setting->secretsFromMap(reply.value().value(setting->name()));
    NetworkManager::VpnSetting::Ptr vpnSetting = connection->settings()->setting(NetworkManager::Setting::SettingType::Vpn).staticCast<NetworkManager::VpnSetting>();
    NMStringMap vpnData = vpnSetting->data();
    QString serviceType = vpnSetting->serviceType();
    if (serviceType == ServiceTypeL2TP) {
        return new L2TPChecker(vpnSetting);
    }
    if (serviceType == ServiceTypePPTP) {
        return new PPTPChecker(vpnSetting);
    }
    if (serviceType == ServiceTypeVPNC) {
        return new VPNCChecker(vpnSetting);
    }
    if (serviceType == ServiceTypeOpenVPN) {
        return new OpenVPNChecker(vpnSetting);
    }
    if (serviceType == ServiceTypeStrongSwan) {
        return new StrongSwanChecker(vpnSetting);
    }
    if (serviceType == ServiceTypeOpenConnect) {
        return new OpenConnectChecker(vpnSetting);
    }
    if (serviceType == ServiceTypeSSTP) {
        return new SSTPChecker(vpnSetting);
    }

    return new DefaultChecker;
}

NMStringMap VPNParametersChecker::data() const
{
    return m_vpnSetting->data();
}

VpnSetting *VPNParametersChecker::setting() const
{
    return m_vpnSetting.data();
}

L2TPChecker::L2TPChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool L2TPChecker::isValid() const
{
    NMStringMap vpnData = data();
    // L2TP中如果网关为空，
    const QString gateWay = vpnData.value("gateway");
    if (gateWay.isEmpty())
        return false;

    // 不支持IPV6
    if (QHostAddress(gateWay).protocol() == QAbstractSocket::IPv6Protocol)
        return false;

    // 用户名不能为空
    if (vpnData.value("user").isEmpty())
        return false;
    //判断密码选项(如果密码为已保存的并且密码为空)
    QString passwordFlags = vpnData.value("password-flags");
    if ((passwordFlags == "0" || passwordFlags.isEmpty())&& setting()->secrets().value("password").isEmpty())
        return false;

    return true;
}

PPTPChecker::PPTPChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool PPTPChecker::isValid() const
{
    NMStringMap vpnData = data();
    // L2TP中如果网关为空，
    const QString gateWay = vpnData.value("gateway");
    if (gateWay.isEmpty())
        return false;

    // 不支持IPV6
    if (QHostAddress(gateWay).protocol() == QAbstractSocket::IPv6Protocol)
        return false;

    // 用户名不能为空
    if (vpnData.value("user").isEmpty())
        return false;
    //判断密码选项(如果密码为已保存的并且密码为空)
    QString passwordFlags = vpnData.value("password-flags");
    if ((passwordFlags == "0" || passwordFlags.isEmpty())&& setting()->secrets().value("password").isEmpty())
        return false;

    return true;
}

VPNCChecker::VPNCChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool VPNCChecker::isValid() const
{
    NMStringMap dataMap = data();
    NMStringMap vpnSettingSecrets = setting()->secrets();
    qInfo() << dataMap << vpnSettingSecrets;
    // 网关不能为空
    if (dataMap.value("IPSec gateway").isEmpty())
        return false;

    // 用户名不能为空
    if (dataMap.value("Xauth username").isEmpty())
        return false;

    // 密码选项如果是已保存的，密码不能为空
    QString passwordFlags = dataMap.value("Xauth password-flags");
    if ((passwordFlags.isEmpty() || passwordFlags == "0") && vpnSettingSecrets.value("Xauth password").isEmpty())
        return false;

    // 组名不能为空
    if (dataMap.value("IPSec ID").isEmpty())
        return false;

    passwordFlags = dataMap.value("IPSec secret-flags");
    if ((passwordFlags.isEmpty() || passwordFlags == "0") && (vpnSettingSecrets.value("IPSec secret").isEmpty()))
        return false;

    return true;
}

OpenVPNChecker::OpenVPNChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{

}

bool OpenVPNChecker::isValid() const
{
    NMStringMap vpnData = data();
    qInfo() << vpnData;
    qInfo() << setting()->secrets();
    if (vpnData.value("remote").isEmpty())
        return false;

    const QString connectionType = vpnData.value("connection-type");
    if (connectionType == "tls") {
        return tlsIsValid();
    }
    if (connectionType == "password") {
        return passIsValid();
    }
    if (connectionType == "password-tls") {
        return passTlsValid();
    }
    if (connectionType == "static-key") {
        return staticKeyIsValid();
    }

    return true;
}

bool OpenVPNChecker::tlsIsValid() const
{
    NMStringMap vpnData = data();
    // 如果是tls的认证方式，则需要判断CA证书
    if (vpnData.value("ca").isEmpty())
        return false;
    // 用户证书
    if (vpnData.value("cert").isEmpty())
        return false;
    // 私钥
    if (vpnData.value("key").isEmpty())
        return false;
    // 密码选项为已保存并且密码为空
    QString certPassFlags = vpnData.value("cert-pass-flags");
    VpnSetting *vpnSetting = setting();
    if ((certPassFlags.isEmpty() || certPassFlags == "0") && vpnSetting && vpnSetting->secrets().value("cert-pass").isEmpty())
        return false;

    return true;
}

bool OpenVPNChecker::passIsValid() const
{
    NMStringMap vpnData = data();
    // 如果是tls的认证方式，则需要判断CA证书
    if (vpnData.value("ca").isEmpty())
        return false;
    // 用户名
    if (vpnData.value("username").isEmpty())
        return false;
    // 密码选项为已保存并且密码为空
    QString passFlags = vpnData.value("password-flags");
    VpnSetting *vpnSetting = setting();
    if ((passFlags.isEmpty() || passFlags == "0") && vpnSetting && vpnSetting->secrets().value("password").isEmpty())
        return false;

    return true;
}

bool OpenVPNChecker::passTlsValid() const
{
    NMStringMap vpnData = data();
    // 如果是tls的认证方式，则需要判断CA证书
    if (vpnData.value("ca").isEmpty())
        return false;
    // 用户名
    if (vpnData.value("username").isEmpty())
        return false;
    // 密码选项为已保存并且密码为空
    QString passFlags = vpnData.value("password-flags");
    VpnSetting *vpnSetting = setting();
    if ((passFlags.isEmpty() || passFlags == "0") && vpnSetting && vpnSetting->secrets().value("password").isEmpty())
        return false;

    // 用户证书
    if (vpnData.value("cert").isEmpty())
        return false;
    // 私钥
    if (vpnData.value("key").isEmpty())
        return false;
    // 密码选项为已保存并且密码为空
    QString certPassFlags = vpnData.value("cert-pass-flags");
    if ((certPassFlags.isEmpty() || certPassFlags == "0") && vpnSetting && vpnSetting->secrets().value("cert-pass").isEmpty())
        return false;

    return true;
}

bool OpenVPNChecker::staticKeyIsValid() const
{
    NMStringMap vpnData = data();
    if (vpnData.value("static-key").isEmpty())
        return false;

    if (vpnData.value("remote-ip").isEmpty())
        return false;

    if (vpnData.value("local-ip").isEmpty())
        return false;

    return true;
}

StrongSwanChecker::StrongSwanChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool StrongSwanChecker::isValid() const
{
    // StrongSwan的vpn只需检测网关是否为空即可
    return !data().value("address").isEmpty();
}

OpenConnectChecker::OpenConnectChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool OpenConnectChecker::isValid() const
{
    NMStringMap dataMap = data();
    // 判断网关是否为空
    if (dataMap.value("gateway").isEmpty())
        return false;

    // 判断用户证书是否为空
    if (dataMap.value("usercert").isEmpty())
        return false;

    // 判断私钥是否为空
    if (dataMap.value("userkey").isEmpty())
        return false;

    return true;
}

SSTPChecker::SSTPChecker(const VpnSetting::Ptr &vpnSetting)
    : VPNParametersChecker(vpnSetting)
{
}

bool SSTPChecker::isValid() const
{
    return true;
}

DefaultChecker::DefaultChecker(bool defaultValue)
    : VPNParametersChecker (nullptr)
    , m_defaultValue(defaultValue)
{
}

bool DefaultChecker::isValid() const
{
    return m_defaultValue;
}
