// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "networkplugin.h"

#include "netmanager.h"
#include "netstatus.h"
#include "netview.h"

#include <DGuiApplicationHelper>

#include <QDBusConnection>
#include <QMouseEvent>
#include <QTime>

#include <unistd.h>

#define STATE_KEY "enabled"

using namespace dde::network;
DGUI_USE_NAMESPACE

namespace dde {
namespace network {
NetworkPlugin::NetworkPlugin(QObject *parent)
    : QObject(parent)
    , m_manager(nullptr)
    , m_netView(nullptr)
    , m_netStatus(nullptr)
    , m_isLockScreen(false)
{
    QTranslator *translator = new QTranslator(this);
    translator->load(QString("/usr/share/dock-network-plugin/translations/dock-network-plugin_%1.qm").arg(QLocale().name()));
    QCoreApplication::installTranslator(translator);
    QDBusConnection::sessionBus().connect("com.deepin.dde.lockFront", "/com/deepin/dde/lockFront", "com.deepin.dde.lockFront", "Visible", this, SLOT(updateLockScreenStatus(bool)));
}

NetworkPlugin::~NetworkPlugin()
{
    if (m_manager)
        delete m_manager;
}

const QString NetworkPlugin::pluginName() const
{
    return "network";
}

const QString NetworkPlugin::pluginDisplayName() const
{
    return tr("Network");
}

void NetworkPlugin::init(PluginProxyInterface *proxyInter)
{
    m_proxyInter = proxyInter;
    if (m_manager)
        return;

    m_manager = new NetManager(this);
    m_netView = new NetView(m_manager);
    m_netStatus = new NetStatus(m_manager);

    m_manager->setServiceLoadForNM(false);
    m_manager->setMonitorNetworkNotify(false);
    m_manager->setAutoAddConnection(false);
    m_manager->setUseSecretAgent(true);
    m_manager->setNetwork8021XMode(NetManager::ToControlCenter);
    m_manager->setCheckPortal(true);
    m_manager->init();

    if (!pluginIsDisable())
        loadPlugin();

    connect(m_netView, &NetView::requestShow, this, &NetworkPlugin::showNetworkDialog);
    connect(m_manager, &NetManager::toControlCenter, this, [=]() {
        m_proxyInter->requestSetAppletVisible(this, NETWORK_KEY, false);
    });
    connect(m_netStatus, &NetStatus::hasDeviceChanged, this, &NetworkPlugin::refreshPluginItemsVisible);
    connect(DGuiApplicationHelper::instance(), &DGuiApplicationHelper::themeTypeChanged, this, &NetworkPlugin::updateIconColor);
}

void NetworkPlugin::invokedMenuItem(const QString &itemKey, const QString &menuId, const bool checked)
{
    Q_UNUSED(checked)

    if (itemKey == NETWORK_KEY)
        m_netStatus->invokeMenuItem(menuId);
}

void NetworkPlugin::refreshIcon(const QString &itemKey)
{
    if (itemKey == NETWORK_KEY)
        m_netStatus->refreshIcon();
}

void NetworkPlugin::pluginStateSwitched()
{
    m_proxyInter->saveValue(this, STATE_KEY, pluginIsDisable());

    refreshPluginItemsVisible();
}

bool NetworkPlugin::pluginIsAllowDisable()
{
    return true;
}

bool NetworkPlugin::pluginIsDisable()
{
    return !m_proxyInter->getValue(this, STATE_KEY, true).toBool();
}

const QString NetworkPlugin::itemCommand(const QString &itemKey)
{
    Q_UNUSED(itemKey)
    if (m_netStatus->needShowControlCenter()) {
        return QString("dbus-send --print-reply "
                       "--dest=com.deepin.dde.ControlCenter "
                       "/com/deepin/dde/ControlCenter "
                       "com.deepin.dde.ControlCenter.ShowModule "
                       "\"string:network\"");
    }

    return QString();
}

const QString NetworkPlugin::itemContextMenu(const QString &itemKey)
{
    if (itemKey == NETWORK_KEY)
        return m_netStatus->contextMenu(true);

    return QString();
}

QWidget *NetworkPlugin::itemWidget(const QString &itemKey)
{
    if (itemKey == NETWORK_KEY) {
        if (m_trayIcon.isNull()) {
            m_trayIcon = m_netStatus->createIconWidget();
            updateIconColor();
            m_trayIcon->installEventFilter(this);
        }
        return m_trayIcon.data();
    }

    return Q_NULLPTR;
}

QWidget *NetworkPlugin::itemTipsWidget(const QString &itemKey)
{
    if (itemKey == NETWORK_KEY && !m_netView->isVisible()) {
        if (m_tipsWidget.isNull())
            m_tipsWidget = m_netStatus->createItemTips();

        return m_tipsWidget.data();
    }

    return Q_NULLPTR;
}

QWidget *NetworkPlugin::itemPopupApplet(const QString &itemKey)
{
    Q_UNUSED(itemKey);
    if (m_netStatus->needShowControlCenter()) {
        return nullptr;
    }
    return m_netView;
}

int NetworkPlugin::itemSortKey(const QString &itemKey)
{
    const QString key = QString("pos_%1_%2").arg(itemKey).arg(Dock::Efficient);
    return m_proxyInter->getValue(this, key, 3).toInt();
}

void NetworkPlugin::setSortKey(const QString &itemKey, const int order)
{
    const QString key = QString("pos_%1_%2").arg(itemKey).arg(Dock::Efficient);
    m_proxyInter->saveValue(this, key, order);
}

void NetworkPlugin::pluginSettingsChanged()
{
    refreshPluginItemsVisible();
}

void NetworkPlugin::loadPlugin()
{
    m_proxyInter->itemAdded(this, NETWORK_KEY);
}

void NetworkPlugin::refreshPluginItemsVisible()
{
    if (pluginIsDisable() || !m_netStatus->hasDevice())
        m_proxyInter->itemRemoved(this, NETWORK_KEY);
    else
        m_proxyInter->itemAdded(this, NETWORK_KEY);
}

bool NetworkPlugin::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type()) {
    case QEvent::ParentChange: {
        if (watched == m_trayIcon && m_trayIcon->parentWidget()) {
            m_trayIcon->parentWidget()->installEventFilter(this);
            m_trayIcon->removeEventFilter(this);
        }
    } break;
    case QEvent::Resize: {
        if (m_trayIcon && watched == m_trayIcon->parentWidget())
            updateIconColor();
    } break;
    default:
        break;
    }
    return QObject::eventFilter(watched, event);
}

void NetworkPlugin::updateLockScreenStatus(bool visible)
{
    m_isLockScreen = visible;
    m_manager->setEnabled(!m_isLockScreen);
}

void NetworkPlugin::updateIconColor()
{
    if (m_trayIcon.isNull() || !m_trayIcon->parentWidget())
        return;

    Qt::GlobalColor color = Qt::white;
    if (DGuiApplicationHelper::instance()->themeType() == DGuiApplicationHelper::LightType && m_trayIcon->parentWidget()->height() <= PLUGIN_BACKGROUND_MIN_SIZE) {
        color = Qt::black;
    }
    QPalette p = m_trayIcon->palette();
    if (p.brightText() != color) {
        p.setColor(QPalette::BrightText, color);
        m_trayIcon->setPalette(p);
    }
}

void NetworkPlugin::showNetworkDialog()
{
    // 第一次连接隐藏网络会回弹，此时锁屏和任务栏都会收到信号
    if (m_isLockScreen || m_trayIcon.isNull() || m_netView->isVisible())
        return;
    m_proxyInter->requestSetAppletVisible(this, NETWORK_KEY, true);
}

} // namespace network
} // namespace dde
