// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "netitem.h"

#include <QApplication>
#include <QDebug>
#include <QThread>

namespace dde {
namespace network {

void NetItemRegisterMetaType()
{
    qRegisterMetaType<NetItemType>("NetItemType");
    qRegisterMetaType<NetConnectionStatus>("NetConnectionStatus");
    qRegisterMetaType<NetDeviceStatus>("NetDeviceStatus");
}

/**
 * 单个列表项的基类
 */
NetItem::NetItem(const QString &id)
    : QObject()
    , m_id(id)
{
    connect(this, &NetItem::objectNameChanged, this, &NetItem::dataChanged);
}

NetItem::~NetItem() { }

QString NetItem::id() const
{
    return m_id;
}

NetItemType NetItem::itemType() const
{
    return NetItemType::RootItem;
}

QVariant NetItem::sortValue() const
{
    return name().toLower();
}

QString NetItem::name() const
{
    return objectName();
}

NetItem *NetItem::getChild(int childPos) const
{
    return (childPos < getChildrenNumber() ? m_children[childPos] : nullptr);
}

int NetItem::getChildIndex(const NetItem *child) const
{
    for (int i = 0; i < m_children.size(); ++i)
        if (m_children[i] == child)
            return static_cast<int>(i);

    return -1;
}

void NetItem::retranslateUi()
{
    for (auto &&item : m_children) {
        item->retranslateUi();
    }
}

bool NetItem::addChild(NetItem *child, int index)
{
    if (!child || m_children.contains(child)) {
        return false;
    }
    if (index < 0 || index >= m_children.size())
        index = m_children.size();

    Q_EMIT childAboutToBeAdded(this, index);
    m_children.insert(m_children.begin() + index, child);

    child->setParent(this);
    Q_EMIT childAdded(child);
    return true;
}

void NetItem::removeChild(NetItem *child)
{
    int pos = getChildIndex(child);
    if (pos >= 0)
        removeChild(pos);
}

void NetItem::removeChild(int index)
{
    if (index < 0 || index >= m_children.size()) {
        return;
    }

    NetItem *child = m_children[index];
    Q_EMIT childAboutToBeRemoved(this, index);
    m_children.erase(m_children.begin() + index);
    child->setParent(nullptr);
    Q_EMIT childRemoved(child);
}

void NetItem::updateName(const QString &name)
{
    if (objectName() != name) {
        setObjectName(name);
        Q_EMIT nameChanged(name);
    }
}

void NetItem::emitDataChanged()
{
    Q_EMIT dataChanged();
}

// 总线控制器
NetControlItem::~NetControlItem() { }

QVariant NetControlItem::sortValue() const
{
    return QString::number(itemType(), 16) + name().toLower();
}

bool NetControlItem::isEnabled() const
{
    return m_enabled;
}

bool NetControlItem::enabledable() const
{
    return m_enabledable;
}

void NetControlItem::updateEnabled(const bool enabled)
{
    if (enabled != m_enabled) {
        m_enabled = enabled;
        Q_EMIT enabledChanged(m_enabled);
    }
}

void NetControlItem::updateEnabledable(const bool enabledable)
{
    if (enabledable != m_enabledable) {
        m_enabledable = enabledable;
        Q_EMIT enabledableChanged(m_enabledable);
    }
}

NetControlItem::NetControlItem(const QString &id)
    : NetItem(id)
    , m_enabled(false)
    , m_enabledable(true)
{
}

// 设备基类
NetDeviceItem::~NetDeviceItem() { }

NetDeviceStatus NetDeviceItem::status() const
{
    return m_status;
}

QStringList NetDeviceItem::ips() const
{
    return m_ips;
}

void NetDeviceItem::updateStatus(const NetDeviceStatus status)
{
    if (status != m_status) {
        m_status = status;
        Q_EMIT statusChanged(m_status);
    }
}

void NetDeviceItem::updateIps(const QStringList &ips)
{
    if (ips != m_ips) {
        m_ips = ips;
        Q_EMIT ipsChanged(m_ips);
    }
}

NetDeviceItem::NetDeviceItem(const QString &id)
    : NetControlItem(id)
    , m_status(NetDeviceStatus::Unknown)

{
}

// 有线设备
NetItemType NetWiredControlItem::itemType() const
{
    return NetItemType::WiredControlItem;
}

void NetWiredControlItem::retranslateUi()
{
    updateName(tr("Wired Network"));
    NetItem::retranslateUi();
}

NetWiredControlItem::NetWiredControlItem(const QString &id)
    : NetControlItem(id)
{
    retranslateUi();
}

// 无线设备
NetItemType NetWirelessControlItem::itemType() const
{
    return NetItemType::WirelessControlItem;
}

void NetWirelessControlItem::retranslateUi()
{
    updateName(tr("Wireless Network"));
    NetItem::retranslateUi();
}

NetWirelessControlItem::NetWirelessControlItem(const QString &id)
    : NetControlItem(id)
{
    retranslateUi();
}

NetItemType NetWiredDeviceItem::itemType() const
{
    return NetItemType::WiredDeviceItem;
}

NetItemType NetWirelessDeviceItem::itemType() const
{
    return NetItemType::WirelessDeviceItem;
}

bool NetWirelessDeviceItem::isApMode() const
{
    return m_isApMode;
}

void NetWirelessDeviceItem::updateApMode(bool apMode)
{
    if (apMode != m_isApMode) {
        m_isApMode = apMode;
        Q_EMIT apModeChanged(m_isApMode);
    }
}

NetWirelessDeviceItem::NetWirelessDeviceItem(const QString &id)
    : NetDeviceItem(id)
    , m_isApMode(false)
{
}

// 有线连接
NetWiredItem::~NetWiredItem() { }

NetItemType NetWiredItem::itemType() const
{
    return NetItemType::WiredItem;
}

NetConnectionStatus NetWiredItem::status() const
{
    return m_status;
}

void NetWiredItem::updateStatus(NetConnectionStatus status)
{
    if (status != m_status) {
        m_status = status;
        Q_EMIT statusChanged(m_status);
    }
}

NetWiredItem::NetWiredItem(const QString &id)
    : NetItem(id)
    , m_status(NetConnectionStatus::UnConnected)
{
}

// 隐藏网络
NetWirelessHiddenItem::~NetWirelessHiddenItem() { }

NetItemType NetWirelessHiddenItem::itemType() const
{
    return NetItemType::WirelessHiddenItem;
}

QVariant NetWirelessHiddenItem::sortValue() const
{
    return "ff";
}

void NetWirelessHiddenItem::retranslateUi()
{
    updateName(tr("Connect to hidden network"));
    NetItem::retranslateUi();
}

NetWirelessHiddenItem::NetWirelessHiddenItem(const QString &id)
    : NetItem(id)
{
    retranslateUi();
}

// 无线网络
NetWirelessItem::~NetWirelessItem() { }

NetItemType NetWirelessItem::itemType() const
{
    return NetItemType::WirelessItem;
}

QVariant NetWirelessItem::sortValue() const
{
    int sortMax = 250;
    switch (m_status) {
    case NetConnectionStatus::Connected:
    case NetConnectionStatus::Connecting:
        sortMax = 140;
        break;
    default:
        break;
    }
    return QString::number(sortMax - m_strength, 16) + name().toLower();
}

uint NetWirelessItem::flags() const
{
    return m_flags;
}

int NetWirelessItem::strength() const
{
    return m_strength;
}

bool NetWirelessItem::isSecure() const
{
    return m_secure;
}

NetConnectionStatus NetWirelessItem::status() const
{
    return m_status;
}

void NetWirelessItem::updateFlags(uint flags)
{
    if (flags != m_flags) {
        m_flags = flags;
        Q_EMIT flagsChanged(m_flags);
    }
}

void NetWirelessItem::updateStrength(int strength)
{
    if (strength != m_strength) {
        m_strength = strength;
        Q_EMIT strengthChanged(m_strength);
        emitDataChanged();
    }
}

void NetWirelessItem::updateSecure(bool secure)
{
    if (secure != m_secure) {
        m_secure = secure;
        Q_EMIT secureChanged(m_secure);
    }
}

void NetWirelessItem::updateStatus(NetConnectionStatus status)
{
    if (status != m_status) {
        m_status = status;
        Q_EMIT statusChanged(m_status);
        emitDataChanged();
    }
}

NetWirelessItem::NetWirelessItem(const QString &id)
    : NetItem(id)
    , m_flags(0)
    , m_strength(0)
    , m_secure(false)
    , m_status(NetConnectionStatus::UnConnected)
{
}

} // namespace network
} // namespace dde
