// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef NETITEM_H
#define NETITEM_H

#include <QObject>
#include <QVariant>

namespace dde {
namespace network {
Q_NAMESPACE

#define NET_WIRED 0x80     // 有线
#define NET_WIRELESS 0x40  // 无线
#define NET_DEVICE 0x20    // 设备
#define NET_CONTROL 0x10   // 控制器
#define NET_ITEM 0x08      // 子项
#define NET_HIDDEN 0x04    // 隐藏网络
#define NET_MASK_ITEM 0x38 // 项掩码
#define NET_MASK_TYPE 0xc0 // 设备类型掩码

enum NetItemType {
    RootItem,                                                  // 根节点
    WirelessControlItem = NET_CONTROL | NET_WIRELESS,          // 无线开关
    WiredControlItem = NET_CONTROL | NET_WIRED,                // 有线开关
    WirelessDeviceItem = NET_DEVICE | NET_WIRELESS,            // 无线设备
    WiredDeviceItem = NET_DEVICE | NET_WIRED,                  // 有线设备
    WirelessItem = NET_ITEM | NET_WIRELESS,                    // 无线项
    WirelessHiddenItem = NET_ITEM | NET_WIRELESS | NET_HIDDEN, // 无线隐藏项
    WiredItem = NET_ITEM | NET_WIRED,                          // 有线项
};

enum class NetConnectionStatus {
    UnConnected = 0, // 未连接
    Connecting,      // 正在连接
    Connected        // 已连接
};

// 位定义用于处理优先级，低位用于同级中区分
enum NetDeviceStatus : unsigned {
    Unknown = 0x000F,           // 未知
    NoCable = 0x001F,           // 无网线
    Enabled = 0x003F,           // 启用
    Disabled = 0x007F,          // 禁用
    ObtainIpFailed = 0x00FF,    // 获取IP失败
    ConnectFailed = 0x01FF,     // 连接失败
    Disconnected = 0x03FF,      // 未连接
    Connected = 0x07FF,         // 已连接
    ConnectNoInternet = 0x0FFF, // 已连接但无网络
    IpConflicted = 0x1FFF,      // IP冲突
    Connecting = 0x3FFF,        // 连接中
    ObtainingIP = 0x3FF1,       // 获取IP
    Authenticating = 0x3FF2,    // 认证中
};

Q_FLAG_NS(NetDeviceStatus);

void NetItemRegisterMetaType();

// 数据项,主线程中(子线程创建,最终会移到主线程中)
class NetItem : public QObject
{
    Q_OBJECT

public:
    virtual ~NetItem();

    QString id() const;
    virtual NetItemType itemType() const;
    virtual QVariant sortValue() const;
    virtual QString name() const;

    virtual NetItem *getChild(int childPos) const;
    virtual int getChildIndex(const NetItem *child) const;

    inline int getChildrenNumber() const { return m_children.size(); }

    inline QList<NetItem *>::const_iterator childBegin() const { return m_children.begin(); }

    inline QList<NetItem *>::const_iterator childEnd() const { return m_children.end(); }

    inline const QList<NetItem *> getchildren() const { return m_children; }

    inline int getIndex() const { return (getParent() ? getParent()->getChildIndex(this) : -1); }

    inline NetItem *getParent() const { return qobject_cast<NetItem *>(parent()); }

    template<class T>
    inline T *toItem()
    {
        return qobject_cast<T *>(this);
    }

public Q_SLOTS:
    virtual void retranslateUi();

protected:
    explicit NetItem(const QString &id);
    virtual bool addChild(NetItem *child, int index = -1);
    virtual void removeChild(NetItem *child);
    virtual void removeChild(int index);
    void updateName(const QString &name);

Q_SIGNALS:
    void nameChanged(const QString &name);
    void dataChanged();
    void childAboutToBeAdded(const NetItem *parent, int pos);
    void childAdded(const NetItem *child);
    void childAboutToBeRemoved(const NetItem *parent, int pos);
    void childRemoved(const NetItem *child);

protected Q_SLOTS:
    void emitDataChanged();

private:
    QString m_id;
    QList<NetItem *> m_children;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetControlItem : public NetItem
{
    Q_OBJECT

public:
    ~NetControlItem() Q_DECL_OVERRIDE;
    QVariant sortValue() const override;

    bool isEnabled() const;
    bool enabledable() const;

Q_SIGNALS:
    void enabledChanged(const bool) const;
    void enabledableChanged(const bool) const;

protected:
    void updateEnabled(const bool enabled);
    void updateEnabledable(const bool enabledable);
    explicit NetControlItem(const QString &id);

private:
    bool m_enabled;
    bool m_enabledable;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetDeviceItem : public NetControlItem
{
    Q_OBJECT

public:
    ~NetDeviceItem() Q_DECL_OVERRIDE;

    NetDeviceStatus status() const;
    QStringList ips() const;

Q_SIGNALS:
    void statusChanged(const NetDeviceStatus status) const;
    void ipsChanged(const QStringList &ips) const;

protected:
    void updateStatus(const NetDeviceStatus status);
    void updateIps(const QStringList &ips);
    explicit NetDeviceItem(const QString &id);

private:
    NetDeviceStatus m_status;
    QStringList m_ips;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWiredControlItem : public NetControlItem
{
    Q_OBJECT

public:
    NetItemType itemType() const override;
    void retranslateUi() override;

protected:
    explicit NetWiredControlItem(const QString &id);

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWirelessControlItem : public NetControlItem
{
    Q_OBJECT

public:
    NetItemType itemType() const override;
    void retranslateUi() override;

protected:
    explicit NetWirelessControlItem(const QString &id);

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWiredDeviceItem : public NetDeviceItem
{
    Q_OBJECT

public:
    NetItemType itemType() const override;

protected:
    using NetDeviceItem::NetDeviceItem;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWirelessDeviceItem : public NetDeviceItem
{
    Q_OBJECT
public:
    NetItemType itemType() const override;
    bool isApMode() const;

Q_SIGNALS:
    void apModeChanged(bool apMode) const;

protected:
    void updateApMode(bool apMode);
    explicit NetWirelessDeviceItem(const QString &id);

private:
    bool m_isApMode;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWiredItem : public NetItem
{
    Q_OBJECT

public:
    ~NetWiredItem() Q_DECL_OVERRIDE;

    NetItemType itemType() const override;
    NetConnectionStatus status() const;

Q_SIGNALS:
    void statusChanged(NetConnectionStatus status);

protected:
    void updateStatus(NetConnectionStatus status);
    explicit NetWiredItem(const QString &id);

private:
    NetConnectionStatus m_status;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWirelessHiddenItem : public NetItem
{
    Q_OBJECT

public:
    ~NetWirelessHiddenItem() Q_DECL_OVERRIDE;
    NetItemType itemType() const override;
    QVariant sortValue() const override;
    void retranslateUi() override;

protected:
    explicit NetWirelessHiddenItem(const QString &id);

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

class NetWirelessItem : public NetItem
{
    Q_OBJECT

    Q_PROPERTY(int strength READ strength NOTIFY strengthChanged)
    Q_PROPERTY(bool secure READ isSecure NOTIFY secureChanged)

public:
    ~NetWirelessItem() Q_DECL_OVERRIDE;

    NetItemType itemType() const override;
    QVariant sortValue() const override;

    uint flags() const;
    int strength() const;
    bool isSecure() const;
    NetConnectionStatus status() const;

Q_SIGNALS:
    void flagsChanged(uint flags);
    void strengthChanged(int strength);
    void secureChanged(bool secure);
    void statusChanged(NetConnectionStatus status);

protected:
    void updateFlags(uint flags);
    void updateStrength(int strength);
    void updateSecure(bool secure);
    void updateStatus(NetConnectionStatus status);

    explicit NetWirelessItem(const QString &id);

private:
    uint m_flags;
    int m_strength;
    bool m_secure;
    NetConnectionStatus m_status;

    friend class NetManagerPrivate;
    friend class NetManagerThreadPrivate;
};

} // namespace network
} // namespace dde

Q_DECLARE_METATYPE(dde::network::NetItemType)
Q_DECLARE_METATYPE(dde::network::NetConnectionStatus)
Q_DECLARE_METATYPE(dde::network::NetDeviceStatus)

#endif //  NETWORKAPPLETMODEL_H
