// SPDX-FileCopyrightText: 2018 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "netmanager.h"

#include "netitem.h"
#include "private/netmanager_p.h"
#include "private/netmanagerthreadprivate.h"

#include <QDebug>
#include <QTimer>

namespace dde {
namespace network {

NetManager::NetManager(QObject *parent)
    : QObject(parent)
    , d_ptrNetManager(new NetManagerPrivate(this))
{
    NetItemRegisterMetaType();
    qRegisterMetaType<NetManager::CmdType>("NetManager::CmdType");
    connect(d_ptrNetManager.get(), &NetManagerPrivate::request, this, &NetManager::request, Qt::QueuedConnection);
}

NetManager::~NetManager() { }

void NetManager::setServiceLoadForNM(bool isNM)
{
    Q_D(NetManager);
    d->setServiceLoadForNM(isNM);
}

void NetManager::setMonitorNetworkNotify(bool monitor)
{
    Q_D(NetManager);
    d->setMonitorNetworkNotify(monitor);
}

void NetManager::setAutoAddConnection(bool autoAdd)
{
    Q_D(NetManager);
    d->setAutoAddConnection(autoAdd);
}

void NetManager::setUseSecretAgent(bool enabled)
{
    Q_D(NetManager);
    d->setUseSecretAgent(enabled);
}

void NetManager::setCheckPortal(bool checkPortal)
{
    Q_D(NetManager);
    d->setCheckPortal(checkPortal);
}

void NetManager::init()
{
    Q_D(NetManager);
    d->init();
}

void NetManager::setNetwork8021XMode(NetManager::Network8021XMode mode)
{
    Q_D(NetManager);
    d->setNetwork8021XMode(mode);
}

void NetManager::setAutoScanInterval(int ms)
{
    Q_D(NetManager);
    d->setAutoScanInterval(ms);
}

void NetManager::setAutoScanEnabled(bool enabled)
{
    Q_D(NetManager);
    d->setAutoScanEnabled(enabled);
}

void NetManager::setEnabled(bool enabled)
{
    Q_D(NetManager);
    d->setEnabled(enabled);
}

NetItem *NetManager::root() const
{
    Q_D(const NetManager);
    return d->m_root;
}

bool NetManager::isAirplaneMode() const
{
    Q_D(const NetManager);
    return d->m_airplaneMode;
}

/**
 * @brief NetManager::exec
 * @param cmd 执行命令 @see NetManager::CmdType
 * @param id 执行对象id
 * @param param 参数
 *
 * @note
 * cmd                  id              param
 * 执行操作 ui=>manager
 * EnabledDevice       netItem.id      空
 * DisabledDevice      netItem.id      空
 * RequestScan         netItem.id      空
 * Disconnect          netItem.id      空
 * Connect             netItem.id      空/QMap<QString, QString> 对应secrets里项 如：{{"identity","uos"},{"password","12345678"}}
 * UserCancelRequest   netItem.id/空   空
 * GoToControlCenter   要跳转的页的page  空   对应控制中心的ShowPage()
 * CheckInput          netItem.id      同Connect参数,实时请求检查
 * 请求交互 manager=>ui
 * RequestPassword     netItem.id      secrets(必需 QStringList)请求的密码，影响显示顺序，
 *                                      prop(非必需 QMap<QString, QString>)id的值(),如：{{"secrets"，{"identity","password"}},{"prop",{"identity","uos"}}
 * CloseInput          netItem.id/空   空
 * InputError          netItem.id      QMap<QString, QString>，对应secrets里各项错误 如：{{"password","密码为空"}}
 * InputValid          netItem.id      同InputError参数,CheckInput的返回
 */
void NetManager::exec(NetManager::CmdType cmd, const QString &id, const QVariantMap &param)
{
    QMetaObject::invokeMethod(d_ptrNetManager.get(), "exec", Qt::QueuedConnection, Q_ARG(NetManager::CmdType, cmd), Q_ARG(QString, id), Q_ARG(QVariantMap, param));
}

void NetManager::setWiredEnabled(bool enabled)
{
    exec(enabled ? NetManager::EnabledDevice : NetManager::DisabledDevice, "NetWiredControlItem");
}

void NetManager::setWirelessEnabled(bool enabled)
{
    exec(enabled ? NetManager::EnabledDevice : NetManager::DisabledDevice, "NetWirelessControlItem");
}

void NetManager::updateLanguage(const QString &local)
{
    Q_EMIT languageChange(local);
}

int NetManager::StrengthLevel(int strength)
{
    if (5 >= strength)
        return 0;

    if (30 >= strength)
        return 20;

    if (55 >= strength)
        return 40;

    if (65 >= strength)
        return 60;

    return 80;
}

///////////////////////////////////////////////////////////////////////
struct PasswordRequest
{
    QString dev;
    QString id;
    QVariantMap param;
    int requestCount;
};

NetManagerPrivate::NetManagerPrivate(NetManager *manager)
    : QObject(manager)
    , m_root(new NetItem("ROOT"))
    , m_airplaneMode(false)
    , m_monitorNetworkNotify(false)
    , m_secretAgent(false)
    , m_autoAddConnection(false)
    , m_managerThread(new NetManagerThreadPrivate)
    , m_passwordRequestData(nullptr)
    , q_ptr(manager)
{
    m_dataMap.insert(m_root->id(), m_root);
    for (int i = 0; i < DeviceItemCount; ++i) {
        m_deviceCount[i] = 0;
    }
    connect(m_managerThread, &NetManagerThreadPrivate::itemAdded, this, &NetManagerPrivate::onItemAdded, Qt::QueuedConnection);
    connect(m_managerThread, &NetManagerThreadPrivate::itemRemoved, this, &NetManagerPrivate::onItemRemoved, Qt::QueuedConnection);
    connect(m_managerThread, &NetManagerThreadPrivate::dataChanged, this, &NetManagerPrivate::onDataChanged, Qt::QueuedConnection);
    connect(m_managerThread, &NetManagerThreadPrivate::request, this, &NetManagerPrivate::sendRequest, Qt::QueuedConnection);
    connect(m_managerThread, &NetManagerThreadPrivate::requestInputPassword, this, &NetManagerPrivate::onRequestPassword, Qt::QueuedConnection);
    connect(m_managerThread, &NetManagerThreadPrivate::networkNotify, q_ptr, &NetManager::networkNotify, Qt::QueuedConnection);
    connect(q_ptr, &NetManager::languageChange, m_managerThread, &NetManagerThreadPrivate::retranslate, Qt::QueuedConnection);
    connect(q_ptr, &NetManager::languageChange, m_root, &NetItem::retranslateUi);
    connect(m_managerThread, &NetManagerThreadPrivate::toControlCenter, q_ptr, &NetManager::toControlCenter, Qt::QueuedConnection);
}

NetManagerPrivate::~NetManagerPrivate()
{
    delete m_managerThread;
    delete m_root;
}

void NetManagerPrivate::setServiceLoadForNM(bool isNM)
{
    m_managerThread->setServiceLoadForNM(isNM);
}

void NetManagerPrivate::setMonitorNetworkNotify(bool monitor)
{
    m_managerThread->setMonitorNetworkNotify(monitor);
}

void NetManagerPrivate::setAutoAddConnection(bool autoAdd)
{
    m_managerThread->setAutoAddConnection(autoAdd);
}

void NetManagerPrivate::setUseSecretAgent(bool enabled)
{
    m_managerThread->setUseSecretAgent(enabled);
}

void NetManagerPrivate::setCheckPortal(bool checkPortal)
{
    m_managerThread->setCheckPortal(checkPortal);
}

void NetManagerPrivate::setNetwork8021XMode(NetManager::Network8021XMode mode)
{
    m_managerThread->setNetwork8021XMode(mode);
}

void NetManagerPrivate::setAutoScanInterval(int ms)
{
    m_managerThread->setAutoScanInterval(ms);
}

void NetManagerPrivate::setAutoScanEnabled(bool enabled)
{
    m_managerThread->setAutoScanEnabled(enabled);
}

void NetManagerPrivate::setEnabled(bool enabled)
{
    m_managerThread->setEnabled(enabled);
}

void NetManagerPrivate::init()
{
    m_managerThread->init();
}

void NetManagerPrivate::exec(NetManager::CmdType cmd, const QString &id, const QVariantMap &param)
{
    qInfo() << "ui request exec cmd:" << cmd << id << param.keys();
    if (!m_showInputId.isEmpty() && m_showInputId != id) {
        // 有显示输入框，新的请求不是原处理
        clearPasswordRequest(m_showInputId);
        sendRequest(NetManager::CloseInput, m_showInputId);
    }
    switch (cmd) {
    case NetManager::DisabledDevice:
    case NetManager::EnabledDevice: {
        NetItem *item = findItem(id);
        NetItemType itemType = NetItemType::RootItem;
        if (item) {
            itemType = item->itemType();
        } else if (id == "NetWirelessControlItem") {
            itemType = NetItemType::WirelessControlItem;
        } else if (id == "NetWiredControlItem") {
            itemType = NetItemType::WiredControlItem;
        }
        switch (itemType) {
        case NetItemType::WiredControlItem:
        case NetItemType::WirelessControlItem: {
            int type = (itemType & NET_MASK_TYPE) | NET_DEVICE;
            int i = m_root->getChildrenNumber();
            while (i--) {
                NetItem *item = m_root->getChild(i);
                if (item->itemType() == type) {
                    setDeviceEnabled(qobject_cast<NetControlItem *>(item), cmd == NetManager::EnabledDevice);
                }
            }
        } break;
        case NetItemType::WiredDeviceItem:
        case NetItemType::WirelessDeviceItem: {
            setDeviceEnabled(qobject_cast<NetControlItem *>(item), cmd == NetManager::EnabledDevice);
        } break;
        default:
            break;
        }
    } break;
    case NetManager::RequestScan: {
        NetItem *item = findItem(id);
        if (item && item->itemType() == NetItemType::WirelessDeviceItem) {
            m_managerThread->requestScan(id);
        }
    } break;
    case NetManager::Disconnect: {
        NetItem *item = findItem(id);
        QString path = id;
        if (item) {
            switch (item->itemType()) {
            case NetItemType::WirelessItem:
            case NetItemType::WiredItem:
                path = item->getParent()->id();
                [[clang::fallthrough]]; // 去无break警告
            case NetItemType::WirelessDeviceItem:
            case NetItemType::WiredDeviceItem: {
                m_managerThread->disconnectDevice(path);
            } break;
            default:
                break;
            }
        }
    } break;
    case NetManager::Connect: {
        const QVariantMap validMap = NetManagerThreadPrivate::CheckParamValid(param);
        if (!validMap.isEmpty()) {
            sendRequest(NetManager::InputError, id, validMap);
            break;
        }
        NetItem *item = findItem(id);
        if (item) {
            switch (item->itemType()) {
            case NetItemType::WirelessHiddenItem: {
                if (param.contains("ssid")) {
                    m_managerThread->connectHidden(item->getParent()->id(), param.value("ssid").toString());
                    sendRequest(NetManager::CloseInput, id);
                } else {
                    QVariantMap param;
                    param.insert("secrets", { "ssid" });
                    sendRequest(NetManager::RequestPassword, id, param);
                }
            } break;
            case NetItemType::WiredItem: {
                NetWiredItem *wiredItem = qobject_cast<NetWiredItem *>(item);
                if (wiredItem->status() == NetConnectionStatus::UnConnected) {
                    m_managerThread->connectWired(id, param);
                }
            } break;
            case NetItemType::WirelessItem: {
                NetWirelessItem *wirelessItem = qobject_cast<NetWirelessItem *>(item);
                if (wirelessItem->status() == NetConnectionStatus::UnConnected || !param.isEmpty()) { // 连接中有可能请求密码
                    m_managerThread->connectWireless(id, param);
                }
            } break;
            default:
                break;
            }
        }
    } break;
    case NetManager::CheckInput: {
        sendRequest(NetManager::InputValid, id, NetManagerThreadPrivate::CheckParamValid(param));
    } break;
    case NetManager::UserCancelRequest: {
        clearPasswordRequest(id);
        sendRequest(NetManager::CloseInput, id);
    } break;
    case NetManager::GoToControlCenter:
        m_managerThread->gotoControlCenter(id);
        break;
    default:
        break;
    }
}

void NetManagerPrivate::onItemAdded(const QString &parentID, NetItem *item)
{
    NetItem *parentItem = findItem(parentID);
    if (!parentItem) {
        qWarning() << __FUNCTION__ << "item Parent not find!" << parentID << item;
        delete item;
        return;
    }
    m_dataMap.insert(item->id(), item);
    parentItem->addChild(item);
    switch (item->itemType()) {
    case NetItemType::WirelessDeviceItem: { // 无线设备添加隐藏网络
        NetWirelessHiddenItem *hiddenItem = new NetWirelessHiddenItem(item->id() + ":Hidden");
        m_dataMap.insert(hiddenItem->id(), hiddenItem);
        item->addChild(hiddenItem);
        ++m_deviceCount[WirelessDeviceIndex];
    } break;
    case NetItemType::WiredDeviceItem: {
        ++m_deviceCount[WiredDeviceIndex];
    } break;
    default:
        return;
    }
    updateControl();
}

void NetManagerPrivate::onItemRemoved(const QString &id)
{
    NetItem *item = findItem(id);
    if (!item) {
        qWarning() << "item removed: item not find!" << id;
        return;
    }

    NetItem *parentItem = item->getParent();
    if (!parentItem) {
        return;
    }
    parentItem->removeChild(item);
    m_dataMap.remove(id);
    item->deleteLater();
    switch (item->itemType()) {
    case NetItemType::WirelessDeviceItem: { // 无线设备添加隐藏网络
        --m_deviceCount[WirelessDeviceIndex];
    } break;
    case NetItemType::WiredDeviceItem: {
        --m_deviceCount[WiredDeviceIndex];
    } break;
    default:
        return;
    }
    updateControl();
}

void NetManagerPrivate::onDataChanged(int dataType, const QString &id, const QVariant &value)
{
    switch (dataType) {
    case NetManagerThreadPrivate::NameChanged: {
        NetItem *item = findItem(id);
        if (item)
            item->updateName(value.toString());
    } break;
    case NetManagerThreadPrivate::EnabledChanged: {
        NetControlItem *item = qobject_cast<NetControlItem *>(findItem(id));
        if (item)
            item->updateEnabled(value.toBool());
        updateControlEnabled(item->itemType());
    } break;
    case NetManagerThreadPrivate::DeviceAvailableChanged: {
        NetControlItem *item = qobject_cast<NetControlItem *>(findItem(id));
        if (item)
            item->updateEnabledable(value.toBool());
        updateControlEnabled(item->itemType());
    } break;
    case NetManagerThreadPrivate::WiredStatusChanged: {
        NetWiredItem *item = qobject_cast<NetWiredItem *>(findItem(id));
        if (item)
            item->updateStatus(value.value<NetConnectionStatus>());
    } break;
    case NetManagerThreadPrivate::WirelessStatusChanged: {
        NetWirelessItem *item = qobject_cast<NetWirelessItem *>(findItem(id));
        if (item)
            item->updateStatus(value.value<NetConnectionStatus>());
    } break;
    case NetManagerThreadPrivate::StrengthChanged: {
        NetWirelessItem *item = qobject_cast<NetWirelessItem *>(findItem(id));
        if (item)
            item->updateStrength(value.toInt());
    } break;
    case NetManagerThreadPrivate::SecuredChanged: {
        NetWirelessItem *item = qobject_cast<NetWirelessItem *>(findItem(id));
        if (item)
            item->updateSecure(value.toBool());
    } break;
    case NetManagerThreadPrivate::IPChanged: {
        NetDeviceItem *item = qobject_cast<NetDeviceItem *>(findItem(id));
        if (item)
            item->updateIps(value.toStringList());
    } break;
    case NetManagerThreadPrivate::DeviceStatusChanged: {
        NetDeviceItem *item = qobject_cast<NetDeviceItem *>(findItem(id));
        if (item)
            item->updateStatus(value.value<NetDeviceStatus>());
    } break;
    case NetManagerThreadPrivate::HotspotEnabledChanged: {
        NetWirelessDeviceItem *item = qobject_cast<NetWirelessDeviceItem *>(findItem(id));
        if (item)
            item->updateApMode(value.toBool());
    } break;
    default:
        break;
    }
}

void NetManagerPrivate::sendRequest(NetManager::CmdType cmd, const QString &id, const QVariantMap &param)
{
    qInfo() << "send request:" << cmd << id << param;
    Q_EMIT request(cmd, id, param);
    switch (cmd) {
    case NetManager::RequestPassword:
        if (!m_showInputId.isEmpty())
            qWarning() << "untreated RequestPassword:" << m_showInputId << "new RequestPassword" << id;
        m_showInputId = id;
        break;
    case NetManager::CloseInput:
        if (id.isEmpty() || id == m_showInputId) {
            m_showInputId.clear();
        }
        break;
    default:
        break;
    }
}

void NetManagerPrivate::onRequestPassword(const QString &dev, const QString &id, const QVariantMap &param)
{
    if (m_passwordRequestData) {
        delete m_passwordRequestData;
        m_passwordRequestData = nullptr;
    }

    if (param.isEmpty()) { // 取消请求
        if (!m_showInputId.isEmpty()) {
            NetItem *item = findItem(m_showInputId);
            if (item && item->name() == id) {
                sendRequest(NetManager::CloseInput, m_showInputId);
            }
        }
        return;
    }

    m_passwordRequestData = new PasswordRequest;
    m_passwordRequestData->dev = dev;
    m_passwordRequestData->id = id;
    m_passwordRequestData->param = param;
    m_passwordRequestData->requestCount = 0;
    findPasswordRequestItem();
}

void NetManagerPrivate::findPasswordRequestItem()
{
    if (!m_passwordRequestData)
        return;
    QString itemId;
    m_passwordRequestData->requestCount++;
    // dev为空时，优先找id相等且为连接中的
    if (m_passwordRequestData->dev.isEmpty()) {
        QList<NetItem *> items;
        items.append(m_root);
        while (!items.isEmpty()) {
            NetItem *item = items.takeFirst();
            switch (item->itemType()) {
            case NetItemType::WirelessItem: {
                NetWirelessItem *wirelessitem = qobject_cast<NetWirelessItem *>(item);
                if (wirelessitem->name() == m_passwordRequestData->id) {
                    // 如果查找15次连接状态还没更新就不判断状态
                    if (m_passwordRequestData->requestCount > 15)
                        itemId = wirelessitem->id();

                    if (wirelessitem->status() == NetConnectionStatus::Connecting) {
                        itemId = wirelessitem->id();
                        break;
                    }
                }
            } break;
            case NetItemType::RootItem:
            case NetItemType::WirelessDeviceItem:
                items.append(item->getchildren());
                break;
            default:
                break;
            }
        }
    } else {
        NetItem *devItem = findItem(m_passwordRequestData->dev);
        if (devItem) {
            switch (devItem->itemType()) {
            case NetItemType::WirelessDeviceItem: {
                for (auto it = devItem->childBegin(); it != devItem->childEnd(); ++it) {
                    if ((*it)->name() == m_passwordRequestData->id) {
                        itemId = (*it)->id();
                        break;
                    }
                }
            } break;
            case NetItemType::WiredDeviceItem:
                break;
            default:
                break;
            }
        }
    }
    if (!itemId.isEmpty()) {
        sendRequest(NetManager::RequestPassword, itemId, m_passwordRequestData->param);
        delete m_passwordRequestData;
        m_passwordRequestData = nullptr;
    } else {
        // 没找到则延时再进入
        QTimer::singleShot(200, this, &NetManagerPrivate::findPasswordRequestItem);
    }
}

void NetManagerPrivate::clearPasswordRequest(const QString &id)
{
    m_managerThread->userCancelRequest(id);
    if (m_passwordRequestData) {
        delete m_passwordRequestData;
        m_passwordRequestData = nullptr;
    }
}

void NetManagerPrivate::setDeviceEnabled(NetControlItem *controlItem, bool enabled)
{
    if (!controlItem)
        return;

    switch (controlItem->itemType()) {
    case NetItemType::WiredControlItem:
    case NetItemType::WirelessControlItem: {
        int type = (controlItem->itemType() & NET_MASK_TYPE) | NET_DEVICE;
        int i = m_root->getChildrenNumber();
        while (i--) {
            NetItem *item = m_root->getChild(i);
            if (item->itemType() == type) {
                setDeviceEnabled(qobject_cast<NetControlItem *>(item), enabled);
            }
        }
    } break;
    case NetItemType::WiredDeviceItem:
    case NetItemType::WirelessDeviceItem: {
        if (controlItem->enabledable() && controlItem->isEnabled() != enabled) {
            m_managerThread->setDeviceEnabled(controlItem->id(), enabled);
        }
    } break;
    default:
        break;
    }
}

void NetManagerPrivate::updateControl()
{
#define UPDATECONTROL(dev)                                                         \
    do {                                                                           \
        NetItem *controlItem = findItem("Net" #dev "ControlItem");                 \
        if (m_deviceCount[dev##DeviceIndex] > 1) {                                 \
            if (!controlItem) {                                                    \
                controlItem = new Net##dev##ControlItem("Net" #dev "ControlItem"); \
                m_dataMap.insert(controlItem->id(), controlItem);                  \
                m_root->addChild(controlItem);                                     \
            }                                                                      \
        } else {                                                                   \
            if (controlItem) {                                                     \
                m_root->removeChild(controlItem);                                  \
                m_dataMap.remove(controlItem->id());                               \
                controlItem->deleteLater();                                        \
            }                                                                      \
        }                                                                          \
    } while (false)

    UPDATECONTROL(Wired);
    UPDATECONTROL(Wireless);
    updateControlEnabled(NetItemType::WiredDeviceItem);
    updateControlEnabled(NetItemType::WirelessDeviceItem);
}

void NetManagerPrivate::updateControlEnabled(int type)
{
    int devType = (type & NET_MASK_TYPE) | NET_DEVICE;
    int parentType = (type & NET_MASK_TYPE) | NET_CONTROL;
    NetControlItem *parentItem = nullptr;
    bool pEnabled = false;
    bool pEnabledable = false;
    for (int i = 0; i < m_root->getChildrenNumber(); ++i) {
        NetItem *item = m_root->getChild(i);
        if (item->itemType() == devType) {
            NetControlItem *dev = qobject_cast<NetDeviceItem *>(item);
            if (dev) {
                pEnabled |= dev->isEnabled();
                pEnabledable |= dev->enabledable();
            }
        } else if (item->itemType() == parentType) {
            parentItem = qobject_cast<NetControlItem *>(item);
        }
    }
    if (parentItem) {
        parentItem->updateEnabled(pEnabled);
        parentItem->updateEnabledable(pEnabledable);
    }
}

void NetManagerPrivate::updateAirplaneMode(bool enabled)
{
    if (m_airplaneMode != enabled) {
        m_airplaneMode = enabled;
        Q_Q(NetManager);
        Q_EMIT q->airplaneModeChanged(m_airplaneMode);
    }
}

} // namespace network
} // namespace dde
