// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#include "netstatus.h"

#include "netitem.h"
#include "netmanager.h"
#include "private/neticonbutton.h"

#include <QDebug>
#include <QEvent>
#include <QJsonDocument>
#include <QTimer>

#define WIRELESS_DEVICE_INDEX 0
#define WIRED_DEVICE_INDEX 1
#define DEVICE_ITEM_COUNT 2

#define HAS_WIRELESS_DEVICE 0x10u
#define HAS_WIRED_DEVICE 0x20u
#define HAS_DEVICE_MASK 0x30u
#define ENABLED_WIRELESS_DEVICE 0x01u
#define ENABLED_WIRED_DEVICE 0x02u
#define ENABLED_DEVICE_MASK 0x03u
#define ENABLEDABLE_WIRELESS_DEVICE 0x04u
#define ENABLEDABLE_WIRED_DEVICE 0x08u
#define ENABLEDABLE_DEVICE_MASK 0x0Cu

namespace dde {
namespace network {

enum MenuItemKey : int {
    MenuSettings = 1,
    MenuEnabled,
    MenuDisabled,
    MenuWiredEnabled,
    MenuWiredDisabled,
    MenuWirelessEnabled,
    MenuWirelessDisabled,
};

NetStatus::NetStatus(NetManager *manager, QObject *parent)
    : QObject(parent)
    , m_manager(manager)
    , m_networkStatus(NetworkStatus::Unknown)
    , m_hasDevice(false)
    , m_deviceFlag(0)
    , m_currentAnimation(0)
    , m_animationTimer(new QTimer(this))
{
    NetItem *root = m_manager->root();
    connect(root, &NetItem::childAdded, this, &NetStatus::onChildAdded);
    connect(root, &NetItem::childRemoved, this, &NetStatus::onChildRemoved);

    connect(this, &NetStatus::networkStatusChanged, this, &NetStatus::updateNetworkTips);
    connect(this, &NetStatus::networkStatusChanged, this, &NetStatus::updateNetworkIcon);
    connect(m_manager, &NetManager::languageChange, this, &NetStatus::updateNetworkTips);

    m_animationTimer->setInterval(250);
    connect(m_animationTimer, &QTimer::timeout, this, &NetStatus::nextAnimation);
}

NetStatus::~NetStatus() { }

void NetStatus::invokeMenuItem(const QString &menuId)
{
    switch (menuId.toInt()) {
    case MenuItemKey::MenuEnabled:
        m_manager->setWiredEnabled(true);
        m_manager->setWirelessEnabled(true);
        break;
    case MenuItemKey::MenuDisabled:
        m_manager->setWiredEnabled(false);
        m_manager->setWirelessEnabled(false);
        break;
    case MenuItemKey::MenuWiredEnabled:
        m_manager->setWiredEnabled(true);
        break;
    case MenuItemKey::MenuWiredDisabled:
        m_manager->setWiredEnabled(false);
        break;
    case MenuItemKey::MenuWirelessEnabled:
        m_manager->setWirelessEnabled(true);
        break;
    case MenuItemKey::MenuWirelessDisabled:
        m_manager->setWirelessEnabled(false);
        break;
    case MenuItemKey::MenuSettings:
        m_manager->exec(NetManager::GoToControlCenter, "");
        break;
    default:
        break;
    }
}

bool NetStatus::needShowControlCenter() const
{
    // 如果没有网络设备，则直接唤起控制中心
    if (!(m_deviceFlag & HAS_DEVICE_MASK))
        return true;
    if (m_deviceFlag & ENABLED_DEVICE_MASK)
        return false;
    return true;
}

const QString NetStatus::contextMenu(bool hasSetting) const
{
    QList<QVariant> items;
    if ((m_deviceFlag & HAS_DEVICE_MASK) == HAS_DEVICE_MASK) { // 无线有线都有
        QMap<QString, QVariant> wireEnable;
        if (m_deviceFlag & ENABLED_WIRED_DEVICE) {
            wireEnable["itemId"] = QString::number(MenuWiredDisabled);
            wireEnable["itemText"] = tr("Disable wired connection");
        } else {
            wireEnable["itemId"] = QString::number(MenuWiredEnabled);
            wireEnable["itemText"] = tr("Enable wired connection");
        }
        wireEnable["isActive"] = bool(m_deviceFlag & ENABLEDABLE_WIRED_DEVICE);
        items.push_back(wireEnable);
        QMap<QString, QVariant> wirelessEnable;
        if (m_deviceFlag & ENABLED_WIRELESS_DEVICE) {
            wirelessEnable["itemText"] = tr("Disable wireless connection");
            wirelessEnable["itemId"] = QString::number(MenuWirelessDisabled);
        } else {
            wirelessEnable["itemText"] = tr("Enable wireless connection");
            wirelessEnable["itemId"] = QString::number(MenuWirelessEnabled);
        }

        wirelessEnable["isActive"] = bool(m_deviceFlag & ENABLEDABLE_WIRELESS_DEVICE);
        items.push_back(wirelessEnable);
    } else if (m_deviceFlag & HAS_DEVICE_MASK) { // 只有一种设备

        QMap<QString, QVariant> enable;
        if (m_deviceFlag & ENABLED_DEVICE_MASK) {
            enable["itemId"] = QString::number(MenuDisabled);
            enable["itemText"] = tr("Disable network");
        } else {
            enable["itemId"] = QString::number(MenuEnabled);
            enable["itemText"] = tr("Enable network");
        }

        enable["isActive"] = bool(m_deviceFlag & ENABLEDABLE_DEVICE_MASK);
        items.push_back(enable);
    }
    if (hasSetting) {
        QMap<QString, QVariant> settings;
        settings["itemId"] = QString::number(MenuSettings);
        settings["itemText"] = tr("Network settings");
        settings["isActive"] = true;
        items.push_back(settings);
    }
    QMap<QString, QVariant> menu;
    menu["items"] = items;
    menu["checkableMenu"] = false;
    menu["singleCheck"] = false;

    return QJsonDocument::fromVariant(menu).toJson();
}

QWidget *NetStatus::createItemTips() const
{
    QLabel *label = new QLabel(m_networkTips);
    connect(this, &NetStatus::networkTipsChanged, label, &QLabel::setText);
    connect(this, &NetStatus::networkTipsChanged, label, &QLabel::adjustSize, Qt::QueuedConnection);
    label->setContentsMargins(10, 0, 10, 0);
    label->setForegroundRole(QPalette::BrightText);
    return label;
}

QWidget *NetStatus::createIconWidget() const
{
    NetIconButton *iconBut = new NetIconButton();
    iconBut->setForegroundRole(QPalette::BrightText);
    iconBut->setIcon(m_networkIcon);
    iconBut->setFixedSize(20, 20);
    connect(this, &NetStatus::networkIconChanged, iconBut, &NetIconButton::setIcon);
    return iconBut;
}

void NetStatus::refreshIcon()
{
    updateStatus();
}

bool NetStatus::hasDevice() const
{
    return m_hasDevice;
}

QString NetStatus::networkTips() const
{
    return m_networkTips;
}

QIcon NetStatus::networkIcon() const
{
    return m_networkIcon;
}

void NetStatus::onChildAdded(const NetItem *child)
{
    const NetDeviceItem *deviceItem = qobject_cast<const NetDeviceItem *>(child);
    connect(deviceItem, &NetDeviceItem::statusChanged, this, &NetStatus::updateStatus);
    connect(deviceItem, &NetDeviceItem::ipsChanged, this, &NetStatus::updateStatus);
    connect(deviceItem, &NetDeviceItem::nameChanged, this, &NetStatus::updateStatus);
    updateStatus();
}

void NetStatus::onChildRemoved(const NetItem *child)
{
    disconnect(child, nullptr, this, nullptr);
    updateStatus();
}

void NetStatus::updateStatus()
{
    NetItem *root = m_manager->root();
    QStringList iphtml[DEVICE_ITEM_COUNT];
    unsigned devStatus[DEVICE_ITEM_COUNT] = { NetDeviceStatus::Unknown, NetDeviceStatus::Unknown };
    m_deviceFlag = 0;
    // 获取各设备状态
    for (auto it = root->childBegin(); it != root->childEnd(); ++it) {
        int index = WIRELESS_DEVICE_INDEX;
        switch ((*it)->itemType()) {
        case NetItemType::WiredDeviceItem:
            index = WIRED_DEVICE_INDEX;
            [[clang::fallthrough]]; // 去无break警告
        case NetItemType::WirelessDeviceItem: {
            NetDeviceItem *devItem = qobject_cast<NetDeviceItem *>(*it);
            if (devItem && devItem->isEnabled() && devItem->status() == NetDeviceStatus::Connected && !devItem->ips().isEmpty()) {
                QStringList ips = devItem->ips();
                if (ips.count() > 3) {
                    ips.erase(ips.begin() + 3, ips.end());
                    ips.append("......");
                }
                iphtml[index] << QString("<tr><td>%1 : </td><td>%2</td></tr>").arg(devItem->name()).arg(ips.join("<br/>"));
            }
            devStatus[index] |= devItem->status();

            m_deviceFlag |= (HAS_WIRELESS_DEVICE << index);
            if (devItem->isEnabled())
                m_deviceFlag |= (ENABLED_WIRELESS_DEVICE << index);
            if (devItem->enabledable())
                m_deviceFlag |= (ENABLEDABLE_WIRELESS_DEVICE << index);

        } break;
        default:
            break;
        }
    }
    iphtml[WIRELESS_DEVICE_INDEX].sort();
    iphtml[WIRED_DEVICE_INDEX].sort();

    unsigned netStatus = devStatus[WIRELESS_DEVICE_INDEX] | devStatus[WIRED_DEVICE_INDEX];
    bool isWirelessStatus = netStatus == devStatus[WIRELESS_DEVICE_INDEX];
    NetworkStatus networkStatus = NetworkStatus::Unknown;
    // 汇总有线无线状态
    // 优先级: 连接中 ip冲突 已连接但无网络 已连接 未连接 禁用 无网线 未知
    switch (netStatus) {
    case NetDeviceStatus::Authenticating:
    case NetDeviceStatus::ObtainingIP:
    case NetDeviceStatus::Connecting:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Connecting;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessConnecting;
        } else {

            networkStatus = NetworkStatus::WiredConnecting;
        }
        break;
    case NetDeviceStatus::IpConflicted:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::WirelessIpConflicted;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessIpConflicted;
        } else {
            networkStatus = NetworkStatus::WiredIpConflicted;
        }
        break;
    case NetDeviceStatus::ConnectNoInternet:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::ConnectNoInternet;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessConnectNoInternet;
        } else {
            networkStatus = NetworkStatus::WiredConnectNoInternet;
        }
        break;
    case NetDeviceStatus::Connected:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Connected;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessConnected;
        } else {
            networkStatus = NetworkStatus::WiredConnected;
        }
        break;
    case NetDeviceStatus::Disconnected:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Disconnected;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessDisconnected;
        } else {
            networkStatus = NetworkStatus::WiredDisconnected;
        }
        break;
    case NetDeviceStatus::ConnectFailed:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Disconnected;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessDisconnected;
        } else {
            networkStatus = NetworkStatus::WiredDisconnected;
        }
        break;
    case NetDeviceStatus::ObtainIpFailed:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Failed;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessFailed;
        } else {
            networkStatus = NetworkStatus::WiredFailed;
        }
        break;
    case NetDeviceStatus::Disabled:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Disabled;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessDisabled;
        } else {
            networkStatus = NetworkStatus::WiredDisabled;
        }
        break;
    case NetDeviceStatus::Enabled:
        if (devStatus[WIRELESS_DEVICE_INDEX] == devStatus[WIRED_DEVICE_INDEX]) {
            networkStatus = NetworkStatus::Disconnected;
        } else if (isWirelessStatus) {
            networkStatus = NetworkStatus::WirelessDisconnected;
        } else {
            networkStatus = NetworkStatus::WiredDisconnected;
        }
        break;
    case NetDeviceStatus::NoCable:
        networkStatus = NetworkStatus::Nocable;
        break;
    case NetDeviceStatus::Unknown:
        networkStatus = NetworkStatus::Unknown;
        break;
    }

    if (m_networkStatus != networkStatus) {
        m_networkStatus = networkStatus;

        bool hasDevice = m_networkStatus != NetworkStatus::Unknown;
        if (m_hasDevice != hasDevice) {
            m_hasDevice = hasDevice;
            Q_EMIT hasDeviceChanged(m_hasDevice);
        }
        Q_EMIT networkStatusChanged(m_networkStatus);
    }
    QString ipTips = iphtml[WIRELESS_DEVICE_INDEX].join(QString()) + iphtml[WIRED_DEVICE_INDEX].join(QString());
    if (ipTips != m_iphtml) {
        m_iphtml = ipTips;
        updateNetworkTips();
    }
    qInfo() << "network status:Wireless:" << NetDeviceStatus(devStatus[WIRELESS_DEVICE_INDEX]) << "+Wired:" << NetDeviceStatus(devStatus[WIRED_DEVICE_INDEX]) << "=" << networkStatus
            << QString::number(m_deviceFlag, 16) << m_iphtml;
}

void NetStatus::updateNetworkTips()
{
    QString networkTips;
    switch (networkStatus()) {
    case NetworkStatus::WirelessConnected:
    case NetworkStatus::WiredConnected:
    case NetworkStatus::Connected: {
        networkTips = m_iphtml;
        if (networkTips.isEmpty())
            networkTips = tr("Connected");
    } break;
    case NetworkStatus::Disabled:
    case NetworkStatus::WirelessDisabled:
    case NetworkStatus::WiredDisabled: {
        networkTips = tr("Device disabled");
    } break;
    case NetworkStatus::Unknown:
    case NetworkStatus::Nocable: {
        networkTips = tr("Network cable unplugged");
    } break;
    case NetworkStatus::Disconnected:
    case NetworkStatus::WirelessDisconnected:
    case NetworkStatus::WiredDisconnected: {
        networkTips = tr("Not connected");
    } break;
    case NetworkStatus::Connecting:
    case NetworkStatus::WirelessConnecting:
    case NetworkStatus::WiredConnecting: {
        networkTips = tr("Connecting");
    } break;
    case NetworkStatus::ConnectNoInternet:
    case NetworkStatus::WirelessConnectNoInternet:
    case NetworkStatus::WiredConnectNoInternet: {
        networkTips = tr("Connected but no Internet access");
    } break;
    case NetworkStatus::Failed:
    case NetworkStatus::WirelessFailed:
    case NetworkStatus::WiredFailed: {
        networkTips = tr("Connection failed");
    } break;
    case NetworkStatus::WiredIpConflicted:
    case NetworkStatus::WirelessIpConflicted: {
        networkTips = tr("IP conflict");
    } break;
    default:
        break;
    }

    if (networkTips != m_networkTips) {
        m_networkTips = networkTips;
        Q_EMIT networkTipsChanged(m_networkTips);
    }
}

void NetStatus::updateNetworkIcon()
{
    QString iconString;
    NetWirelessItem *activeItem = nullptr;
    m_animationIcon.clear();
    switch (networkStatus()) {
    case NetworkStatus::Disabled:
    case NetworkStatus::WirelessDisabled:
        iconString = "wireless-disabled-symbolic";
        break;
    case NetworkStatus::WiredDisabled:
        iconString = "network-disabled-symbolic";
        break;
    case NetworkStatus::Connected:
    case NetworkStatus::WirelessConnected: {
        NetWirelessItem *item = getConnectWirelessItem();
        if (item) {
            activeItem = item;
            iconString = QString("wireless%1-%2-symbolic").arg(item->flags() ? "6" : "").arg(NetManager::StrengthLevel(item->strength()));
        } else {
            QTimer::singleShot(500, this, &NetStatus::updateNetworkIcon);
            iconString = "wireless-80-symbolic";
        }
    } break;
    case NetworkStatus::WiredConnected:
        iconString = "network-online-symbolic";
        break;
    case NetworkStatus::Disconnected:
    case NetworkStatus::WirelessDisconnected:
        iconString = "wireless-0-symbolic";
        break;
    case NetworkStatus::WiredDisconnected:
        iconString = "network-none-symbolic";
        break;
    case NetworkStatus::Connecting: {
        m_animationIcon = QStringList({
                "network-wired-symbolic-connecting1",
                "network-wired-symbolic-connecting2",
                "network-wired-symbolic-connecting3",
                "network-wired-symbolic-connecting4",
                "network-wired-symbolic-connecting5",
                "network-wired-symbolic-connecting1",
                "network-wired-symbolic-connecting2",
                "network-wired-symbolic-connecting3",
                "network-wired-symbolic-connecting4",
                "network-wired-symbolic-connecting5",
                "wireless-0-symbolic",
                "wireless-20-symbolic",
                "wireless-40-symbolic",
                "wireless-60-symbolic",
                "wireless-80-symbolic",
                "wireless-0-symbolic",
                "wireless-20-symbolic",
                "wireless-40-symbolic",
                "wireless-60-symbolic",
                "wireless-80-symbolic",
        });
        iconString = m_animationIcon.first();
    } break;
    case NetworkStatus::WirelessConnecting: {
        m_animationIcon = QStringList({
                "wireless-0-symbolic",
                "wireless-20-symbolic",
                "wireless-40-symbolic",
                "wireless-60-symbolic",
                "wireless-80-symbolic",
        });
        iconString = m_animationIcon.first();
    } break;
    case NetworkStatus::WiredConnecting: {
        m_animationIcon = QStringList({
                "network-wired-symbolic-connecting1",
                "network-wired-symbolic-connecting2",
                "network-wired-symbolic-connecting3",
                "network-wired-symbolic-connecting4",
                "network-wired-symbolic-connecting5",
        });
        iconString = m_animationIcon.first();
    } break;
    case NetworkStatus::ConnectNoInternet:
    case NetworkStatus::WirelessConnectNoInternet: {
        // 无线已连接但无法访问互联网 offline
        NetWirelessItem *item = getConnectWirelessItem();
        if (item) {
            activeItem = item;
            iconString = QString("wireless%1-offline-symbolic").arg(item->flags() ? "6" : "");
        } else {
            QTimer::singleShot(500, this, &NetStatus::updateNetworkIcon);
            iconString = "wireless-offline-symbolic";
        }
        break;
    }
    case NetworkStatus::WiredConnectNoInternet: {
        iconString = "network-warning-symbolic";
        break;
    }
    case NetworkStatus::WiredFailed: {
        // 有线连接失败none变为offline
        iconString = "network-offline-symbolic";
        break;
    }
    case NetworkStatus::Unknown:
    case NetworkStatus::Nocable: {
        // 待图标 暂用错误图标
        iconString = "network-error-symbolic";
        break;
    }
    case NetworkStatus::WirelessIpConflicted: {
        iconString = "wireless-offline-symbolic";
        break;
    }
    case NetworkStatus::WiredIpConflicted: {
        iconString = "network-offline-symbolic";
        break;
    }
    case NetworkStatus::WirelessFailed:
    case NetworkStatus::Failed: {
        // 无线连接失败改为 disconnect
        iconString = "wireless-disconnect";
        break;
    }
    default:
        iconString = "wireless-disconnect";
        break;
    }
    if (m_activeItem != activeItem) {
        if (m_activeItem)
            disconnect(m_activeItem, nullptr, this, nullptr);
        m_activeItem = activeItem;
        if (m_activeItem) {
            connect(m_activeItem, &NetWirelessItem::strengthChanged, this, &NetStatus::updateNetworkIcon);
            connect(m_activeItem, &NetWirelessItem::destroyed, this, &NetStatus::updateNetworkIcon);
        }
    }
    if (m_animationIcon.isEmpty()) {
        m_animationTimer->stop();
    } else {
        m_animationTimer->start();
    }
    updateIconStr(iconString);
}

void NetStatus::updateIconStr(const QString &iconstr)
{
    if (iconstr != m_networkIconStr) {
        m_networkIconStr = iconstr;
        m_networkIcon = QIcon::fromTheme(m_networkIconStr);
        Q_EMIT networkIconChanged(m_networkIcon);
    }
}

void NetStatus::nextAnimation()
{
    ++m_currentAnimation;
    if (m_currentAnimation < 0 || m_currentAnimation >= m_animationIcon.count())
        m_currentAnimation = 0;
    updateIconStr(m_animationIcon.at(m_currentAnimation));
}

NetWirelessItem *NetStatus::getConnectWirelessItem()
{
    NetWirelessItem *retitem = nullptr;
    QList<NetItem *> items;
    items.append(m_manager->root());
    while (!items.isEmpty()) {
        NetItem *item = items.takeFirst();
        switch (item->itemType()) {
        case NetItemType::WirelessItem: {
            NetWirelessItem *wirelessitem = qobject_cast<NetWirelessItem *>(item);
            switch (wirelessitem->status()) {
            case NetConnectionStatus::Connecting:
                retitem = wirelessitem;
                break;
            case NetConnectionStatus::Connected: {
                if (!retitem || (retitem->status() == NetConnectionStatus::Connected && wirelessitem->strength() > retitem->strength()))
                    retitem = wirelessitem;
            } break;
            default:
                break;
            }
        } break;
        case NetItemType::RootItem:
        case NetItemType::WirelessDeviceItem:
            items.append(item->getchildren());
            break;
        default:
            break;
        }
    }
    return retitem;
}

} // namespace network
} // namespace dde
