// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#include "netview.h"

#include "netitem.h"
#include "netmanager.h"
#include "private/netdelegate.h"
#include "private/netmodel.h"

#include <DStyle>
#include <DStyleOption>

#include <QEvent>
#include <QHoverEvent>
#include <QScrollBar>
#include <QScroller>
#include <QSortFilterProxyModel>
#include <QTimer>

DWIDGET_USE_NAMESPACE

namespace dde {
namespace network {

NetView::NetView(NetManager *manager)
    : QTreeView(nullptr)
    , m_manager(manager)
    , m_closeOnClear(true)
{
    setAccessibleName("tree_network");
    setForegroundRole(QPalette::BrightText);
    setFrameShape(QFrame::NoFrame);

    m_model = new NetModel(this);
    m_model->setRoot(m_manager->root());

    m_proxyModel = new QSortFilterProxyModel(m_model);
    m_proxyModel->setSortRole(SORTROLE);
    m_proxyModel->setSourceModel(m_model);

    setModel(m_proxyModel);
    sortByColumn(0, Qt::AscendingOrder);

    m_delegate = new NetDelegate(m_proxyModel, this);
    setItemDelegate(m_delegate);

    connect(m_delegate, &NetDelegate::requestSetEnabled, this, &NetView::onRequestSetEnabled);
    connect(m_delegate, &NetDelegate::requestScan, this, &NetView::onRequestScan);
    connect(m_delegate, &NetDelegate::requestDisconnect, this, &NetView::onRequestDisconnect);
    connect(m_delegate, &NetDelegate::requestCheckInput, this, &NetView::onRequestCheckInput);
    connect(m_delegate, &NetDelegate::requestConnect, this, &NetView::onRequestConnect);
    connect(m_delegate, &NetDelegate::requestUpdateLayout, this, &NetView::updateLayout, Qt::QueuedConnection);
    connect(m_delegate, &NetDelegate::requestShow, this, &NetView::scrollToItem, Qt::QueuedConnection);

    setFixedWidth(300);

    setFrameShape(QFrame::NoFrame);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

    setSelectionMode(QAbstractItemView::NoSelection);
    setRootIsDecorated(false);
    setItemsExpandable(false);
    setHeaderHidden(true);
    setAnimated(false);
    setIndentation(0);
    expandAll();

    viewport()->setAutoFillBackground(false);

    connect(this, &NetView::clicked, this, &NetView::activated);
    connect(this, &NetView::activated, this, &NetView::onActivated);

    connect(m_manager, &NetManager::request, this, &NetView::onRequest);

    // 支持在触摸屏上滚动
    QScroller::grabGesture(viewport(), QScroller::LeftMouseButtonGesture);
    QScrollerProperties sp;
    sp.setScrollMetric(QScrollerProperties::VerticalOvershootPolicy, QScrollerProperties::OvershootAlwaysOff);
}

NetView::~NetView() = default;

void NetView::setForegroundRole(QPalette::ColorRole role)
{
    QTreeView::setForegroundRole(role);
    viewport()->setForegroundRole(role);
}

bool NetView::closeOnClear() const
{
    return m_closeOnClear;
}

void NetView::setCloseOnClear(bool closeOnClear)
{
    m_closeOnClear = closeOnClear;
}

void NetView::clear()
{
    if (isVisible())
        return;

    m_manager->exec(NetManager::UserCancelRequest, "");
    scrollTo(model()->index(0, 0));
}

void NetView::rowsInserted(const QModelIndex &parent, int start, int end)
{
    QAbstractItemModel *m = model();
    QList<QModelIndex> indexes;
    indexes.append(parent);
    while (!indexes.isEmpty()) {
        QModelIndex i = indexes.takeFirst();
        if (!isPersistentEditorOpen(i))
            openPersistentEditor(i);
        for (int j = 0; j < m->rowCount(i); j++) {
            indexes.append(m->index(j, 0, i));
        }
    }
    QTreeView::rowsInserted(parent, start, end);

    QModelIndex newIndex = m->index(start, 0, parent);
    QModelIndex sourceIndex = m_proxyModel->mapToSource(newIndex);
    NetItem *item = m_model->toObject(sourceIndex);

    if (!item)
        return;
    NetDeviceItem *dev = item->toItem<NetDeviceItem>();
    if (dev) {
        updateItemExpand(dev);
        connect(dev, &NetDeviceItem::enabledChanged, this, &NetView::onExpandStatusChanged);
        if (dev->itemType() == NetItemType::WirelessDeviceItem)
            connect(dev->toItem<NetWirelessDeviceItem>(), &NetWirelessDeviceItem::apModeChanged, this, &NetView::onExpandStatusChanged);
    }
}

void NetView::drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    QStyleOptionViewItem opt = option;
    opt.state |= QStyle::State_Active;

    bool drawLine = false;
    DStyleOptionBackgroundGroup boption;
    boption.init(this);
    boption.QStyleOption::operator=(opt);

    if (opt.rect.contains(QPoint(0, height() - 1))) {
        if (opt.rect.contains(QPoint(0, 1)))
            opt.viewItemPosition = QStyleOptionViewItem::OnlyOne;
        else
            opt.viewItemPosition = QStyleOptionViewItem::End;
    } else if (opt.rect.contains(QPoint(0, 1))) {
        opt.viewItemPosition = QStyleOptionViewItem::Beginning;
    } else {
        opt.viewItemPosition = QStyleOptionViewItem::Middle;
    }
    boption.directions = Qt::Vertical;
    boption.position = DStyleOptionBackgroundGroup::ItemBackgroundPosition(opt.viewItemPosition);

    switch (index.data(TYPEROLE).value<NetItemType>()) {
    case NetItemType::WirelessControlItem:
    case NetItemType::WiredControlItem:
    case NetItemType::WirelessDeviceItem:
    case NetItemType::WiredDeviceItem:
        drawLine = true;
        break;
    default: {
        if (currentIndex() == index) {
            QColor color(option.palette.brightText().color());
            color.setAlphaF(0.2);
            boption.dpalette.setBrush(DPalette::ItemBackground, color);
            style()->drawPrimitive(static_cast<QStyle::PrimitiveElement>(DStyle::PE_ItemBackground), &boption, painter, opt.widget);
        }
    } break;
    }

    if (opt.viewItemPosition == QStyleOptionViewItem::End) {
        drawLine = false;
    } else if (!drawLine) {
        drawLine = index.data(TYPEROLE).value<NetItemType>() == NetItemType::WirelessHiddenItem
                || index.siblingAtRow(index.row() + 1).data(TYPEROLE).value<NetItemType>() == NetItemType::WirelessHiddenItem;
    }
    if (drawLine) {
        // 绘制间隔线
        QRect rct = option.rect;
        rct.setY(rct.top() + rct.height() - 2);
        rct.setHeight(2);
        QColor color(option.palette.brightText().color());
        color.setAlphaF(0.1);
        painter->fillRect(rct, color);
    }
    QTreeView::drawRow(painter, opt, index);
}

bool NetView::viewportEvent(QEvent *event)
{
    switch (event->type()) {
    case QEvent::HoverLeave: {
        setCurrentIndex(QModelIndex());
    } break;
    case QEvent::HoverEnter:
    case QEvent::HoverMove: {
        QHoverEvent *he = static_cast<QHoverEvent *>(event);
        QModelIndex newIndex = indexAt(he->pos());
        setCurrentIndex(newIndex);
        break;
    }
    default:
        return QTreeView::viewportEvent(event);
    }
    return true;
}

void NetView::mousePressEvent(QMouseEvent *event)
{
    QModelIndex index = indexAt(event->pos());
    if (!index.isValid() || !index.flags().testFlag(Qt::ItemIsEnabled)) {
        QAbstractItemView::mousePressEvent(event);
    } else {
        QTreeView::mousePressEvent(event);
    }
}

void NetView::mouseReleaseEvent(QMouseEvent *event)
{
    QModelIndex index = indexAt(event->pos());
    if (!index.isValid() || !index.flags().testFlag(Qt::ItemIsEnabled)) {
        QAbstractItemView::mouseReleaseEvent(event);
    } else {
        QTreeView::mouseReleaseEvent(event);
    }
}

void NetView::updateByScrollbar()
{
    if (m_updateCurrent) {
        QPoint posInVp = viewport()->mapFromGlobal(QCursor::pos());
        if (viewport()->rect().contains(posInVp))
            setCurrentIndex(indexAt(posInVp));
    }
    m_updateCurrent = true;
}

void NetView::onRequestSetEnabled(const QString &id, bool enabled)
{
    m_manager->exec(enabled ? NetManager::EnabledDevice : NetManager::DisabledDevice, id);
}

void NetView::onRequestScan(const QString &id)
{
    m_manager->exec(NetManager::RequestScan, id);
}

void NetView::onRequestDisconnect(const QString &id)
{
    m_manager->exec(NetManager::Disconnect, id);
}

void NetView::onRequestCheckInput(const QString &id, const QVariantMap &param)
{
    m_manager->exec(NetManager::CheckInput, id, param);
}

void NetView::onRequestConnect(const QString &id, const QVariantMap &param)
{
    if (param.value("input", true).toBool()) {
        m_manager->exec(NetManager::Connect, id, param);
    } else {
        m_manager->exec(NetManager::UserCancelRequest, id, param);
    }
}

void NetView::onActivated(const QModelIndex &index)
{
    m_manager->exec(NetManager::Connect, index.data(IDROLE).toString());
}

void NetView::onRequest(NetManager::CmdType cmd, const QString &id, const QVariantMap &param)
{
    qInfo() << "request ui:" << cmd << id << param;
    switch (cmd) {
    case NetManager::RequestPassword: {
        Q_EMIT requestShow();
        m_delegate->showPassword(id, param);
    } break;
    case NetManager::InputError: {
        m_delegate->showError(id, param);
    } break;
    case NetManager::InputValid: {
        m_delegate->updateInputValid(id, param);
    } break;
    case NetManager::CloseInput: {
        m_delegate->closeInput();
    } break;
    default:
        break;
    }
}

void NetView::updateLayout()
{
    scheduleDelayedItemsLayout();
    if (!isVisible())
        updateGeometries();
}

void NetView::onExpandStatusChanged()
{
    NetDeviceItem *dev = qobject_cast<NetDeviceItem *>(sender());
    if (dev)
        updateItemExpand(dev);
}

void NetView::updateItemExpand(NetItem *item)
{
    bool expandItem = false;
    switch (item->itemType()) {
    case NetItemType::WiredDeviceItem: {
        NetWiredDeviceItem *dev = item->toItem<NetWiredDeviceItem>();
        expandItem = dev->isEnabled();
    } break;
    case NetItemType::WirelessDeviceItem: { // 无线禁用或热点模式时折叠
        NetWirelessDeviceItem *dev = item->toItem<NetWirelessDeviceItem>();
        expandItem = dev->isEnabled() && !dev->isApMode();
    } break;
    default:
        return;
    }
    QModelIndex sIndex = m_model->index(item);
    QModelIndex index = m_proxyModel->mapFromSource(sIndex);
    if (isExpanded(index) != expandItem) {
        setExpanded(index, expandItem);
        updateGeometries();
        viewport()->update();
    }
}

void NetView::scrollToItem(const QString &id)
{
    QAbstractItemModel *m = model();
    QList<QModelIndex> indexes;
    indexes.append(QModelIndex());
    while (!indexes.isEmpty()) {
        QModelIndex index = indexes.takeFirst();
        if (index.data(IDROLE).toString() == id) {
            scrollTo(index);
            return;
        }
        if (!index.isValid() || isExpanded(index)) {
            int row = m->rowCount(index);
            while (row--) {
                indexes.prepend(m->index(row, 0, index));
            }
        }
    }
}

QModelIndex NetView::traverseAndSearch(const QModelIndex &parent, const QString &id)
{
    int row = m_proxyModel->rowCount(parent);
    while (row--) {
        QModelIndex index = m_proxyModel->index(row, 0, parent);
        if (index.isValid() && index.data(IDROLE).toString() == id) {
            return index;
        }
        QModelIndex retIndex = traverseAndSearch(index, id);
        if (retIndex.isValid())
            return retIndex;
    }
    return QModelIndex();
}

void NetView::verticalScrollbarValueChanged(int)
{
    QMetaObject::invokeMethod(this, "updateByScrollbar", Qt::QueuedConnection);
}

void NetView::horizontalScrollbarValueChanged(int)
{
    QMetaObject::invokeMethod(this, "updateByScrollbar", Qt::QueuedConnection);
}

void NetView::currentChanged(const QModelIndex &current, const QModelIndex &previous)
{
    QTreeView::currentChanged(current, previous);
    if (previous.isValid()) {
        QRect rect = visualRect(previous);
        rect.setRect(0, rect.y() - 1, viewport()->width(), rect.height() + 2);
        viewport()->update(rect);
    }
    if (current.isValid()) {
        QRect rect = visualRect(current);
        rect.setRect(0, rect.y() - 1, viewport()->width(), rect.height() + 2);
        viewport()->update(rect);
    }
    m_updateCurrent = false;
}

void NetView::updateGeometries()
{
    QTreeView::updateGeometries();

    // 最多显示16项
    QAbstractItemModel *m = model();
    QList<QModelIndex> indexes;
    indexes.append(QModelIndex());
    int count = 0;
    int h = 0;
    while (!indexes.isEmpty()) {
        QModelIndex index = indexes.takeFirst();
        if (index.isValid()) {
            h += rowHeight(index);
        }
        count++;
        if (count > 16) {
            break;
        }
        if (!index.isValid() || isExpanded(index)) {
            int row = m->rowCount(index);
            while (row--) {
                indexes.prepend(m->index(row, 0, index));
            }
        }
    }
    setFixedHeight(h);
}

void NetView::showEvent(QShowEvent *event)
{
    QTreeView::showEvent(event);
    m_manager->setAutoScanEnabled(true);
}

void NetView::hideEvent(QHideEvent *event)
{
    QTreeView::hideEvent(event);
    if (closeOnClear()) {
        // 延时调用,防止密码拉起时收到多个hide事件,误触发clear,
        QTimer::singleShot(10, this, &NetView::clear);
    }
    m_manager->setAutoScanEnabled(false);
}

void NetView::keyPressEvent(QKeyEvent *event)
{
    switch (event->key()) {
    case Qt::Key_Space:
    case Qt::Key_Select:
    case Qt::Key_O:
    case Qt::Key_Enter:
    case Qt::Key_Return: {
        QAbstractItemModel *m = model();
        QList<QModelIndex> indexes;
        indexes.append(QModelIndex());

        while (!indexes.isEmpty()) {
            QModelIndex index = indexes.takeFirst();
            if (index.isValid() && rowHeight(index) > 80) { // 输入框展开状态跳过键盘事件
                return;
            }
            if (!index.isValid() || isExpanded(index)) {
                int row = m->rowCount(index);
                while (row--) {
                    indexes.prepend(m->index(row, 0, index));
                }
            }
        }
    } break;
    default:
        break;
    }
    QTreeView::keyPressEvent(event);
}

} // namespace network
} // namespace dde
