// SPDX-FileCopyrightText: 2019 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later
#include "netdelegate.h"

#include "neticonbutton.h"
#include "netitem.h"
#include "netmanager.h"
#include "netsecretwidget.h"

#include <DIconButton>
#include <DLabel>
#include <DSpinner>
#include <DStyleOption>
#include <DSwitchButton>

#include <QHBoxLayout>
#include <QMouseEvent>
#include <QPainter>
#include <QSortFilterProxyModel>
#include <QTimer>

DWIDGET_USE_NAMESPACE

#define MAX_TEXT_WIDTH 180

namespace dde {
namespace network {

NetDelegate::NetDelegate(const QSortFilterProxyModel *model, QObject *parent)
    : QStyledItemDelegate(parent)
    , m_model(model)
{
}

NetDelegate::~NetDelegate() { }

void NetDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    // 覆盖基类
    Q_UNUSED(painter)
    Q_UNUSED(option)
    Q_UNUSED(index)
}

QWidget *NetDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(option)
    NetItem *item = static_cast<NetItem *>(m_model->mapToSource(index).internalPointer());
    NetWidget *netWidget = nullptr;
    if (item) {
        switch (item->itemType()) {
        case NetItemType::WirelessControlItem:
        case NetItemType::WiredControlItem:
        case NetItemType::WirelessDeviceItem:
        case NetItemType::WiredDeviceItem: {
            NetDeviceWidget *w = new NetDeviceWidget(static_cast<NetDeviceItem *>(item), parent);
            connect(w, &NetDeviceWidget::requestSetEnabled, this, &NetDelegate::requestSetEnabled);
            connect(w, &NetDeviceWidget::requestScan, this, &NetDelegate::requestScan);
            netWidget = w;
            break;
        }
        case NetItemType::WirelessItem: {
            NetWirelessWidget *w = new NetWirelessWidget(static_cast<NetWirelessItem *>(item), parent);
            connect(w, &NetWirelessWidget::requestDisconnect, this, &NetDelegate::requestDisconnect);
            netWidget = w;
            break;
        }
        case NetItemType::WirelessHiddenItem: {
            netWidget = new NetWirelessHiddenWidget(static_cast<NetWirelessHiddenItem *>(item), parent);
            break;
        }
        case NetItemType::WiredItem: {
            NetWiredWidget *w = new NetWiredWidget(static_cast<NetWiredItem *>(item), parent);
            connect(w, &NetWiredWidget::requestDisconnect, this, &NetDelegate::requestDisconnect);
            netWidget = w;
            break;
        }
        default:
            qWarning() << "error NetItemType:" << item->itemType() << item;
            return new QLabel(item->name(), parent);
        }
        if (netWidget) {
            connect(netWidget, &NetWidget::requestConnect, this, &NetDelegate::requestConnect);
            connect(netWidget, &NetWidget::requestCheckInput, this, &NetDelegate::requestCheckInput);
            connect(this, &NetDelegate::requestShowPassword, netWidget, &NetWidget::showPassword);
            connect(this, &NetDelegate::requestShowError, netWidget, &NetWidget::showError);
            connect(this, &NetDelegate::requestUpdateInputValid, netWidget, &NetWidget::updateInputValid);
            connect(this, &NetDelegate::requestCloseInput, netWidget, &NetWidget::closeInput);
        }
    }
    return netWidget;
}

void NetDelegate::showPassword(const QString &id, const QVariantMap &param)
{
    Q_EMIT requestShowPassword(id, param);
    Q_EMIT requestUpdateLayout();
    Q_EMIT requestShow(id);
}

void NetDelegate::showError(const QString &id, const QVariantMap &param)
{
    Q_EMIT requestShowError(id, param);
}

void NetDelegate::updateInputValid(const QString &id, const QVariantMap &param)
{
    Q_EMIT requestUpdateInputValid(id, param);
}

void NetDelegate::closeInput()
{
    Q_EMIT requestCloseInput();
    Q_EMIT requestUpdateLayout();
}

NetWidget::NetWidget(NetItem *item, QWidget *parent)
    : QWidget(parent)
    , m_item(item)
    , m_mainLayout(new QVBoxLayout(this))
    , m_noMousePropagation(false)
{
    m_mainLayout->setContentsMargins(0, 0, 0, 0);
    if (parent)
        setForegroundRole(parent->foregroundRole());
}

NetWidget::~NetWidget() { }

void NetWidget::setCentralWidget(QWidget *widget)
{
    m_mainLayout->addWidget(widget);
}

void NetWidget::addPasswordWidget(QWidget *widget)
{
    m_mainLayout->addWidget(widget);
}

void NetWidget::setNoMousePropagation(bool noMousePropagation)
{
    m_noMousePropagation = noMousePropagation;
}

void NetWidget::removePasswordWidget()
{
    if (m_mainLayout->count() == 2) {
        setNoMousePropagation(false);
        QLayoutItem *item = m_mainLayout->takeAt(1);
        QWidget *w = item->widget();
        if (w)
            delete w;
        delete item;
    }
}

void NetWidget::showPassword(const QString &id, const QVariantMap &param)
{
    if (id != m_item->id() || m_mainLayout->count() >= 2)
        return;
    setNoMousePropagation(true);
    NetSecretWidget *secretWidget = new NetSecretWidget(this);
    secretWidget->initUI(param);
    addPasswordWidget(secretWidget);
    connect(secretWidget, &NetSecretWidget::submit, this, &NetWidget::onSubmit);
    connect(secretWidget, &NetSecretWidget::requestCheckInput, this, &NetWidget::onRequestCheckInput);
}

void NetWidget::showError(const QString &id, const QVariantMap &param)
{
    if (id != m_item->id() || m_mainLayout->count() != 2)
        return;
    NetSecretWidget *secretWidget = qobject_cast<NetSecretWidget *>(m_mainLayout->itemAt(1)->widget());
    if (secretWidget)
        secretWidget->showError(param);
}

void NetWidget::onRequestCheckInput(const QVariantMap &param)
{
    Q_EMIT requestCheckInput(m_item->id(), param);
}

void NetWidget::updateInputValid(const QString &id, const QVariantMap &param)
{
    if (id != m_item->id() || m_mainLayout->count() != 2)
        return;
    NetSecretWidget *secretWidget = qobject_cast<NetSecretWidget *>(m_mainLayout->itemAt(1)->widget());
    if (secretWidget)
        secretWidget->updateInputValid(param);
}

void NetWidget::onSubmit(const QVariantMap &param)
{
    Q_EMIT requestConnect(m_item->id(), param);
}

void NetWidget::closeInput()
{
    removePasswordWidget();
}

void NetWidget::mousePressEvent(QMouseEvent *event)
{
    if (!m_noMousePropagation)
        QWidget::mousePressEvent(event);
}

void NetWidget::mouseReleaseEvent(QMouseEvent *event)
{
    if (!m_noMousePropagation)
        QWidget::mousePressEvent(event);
}

NetDeviceWidget::NetDeviceWidget(NetDeviceItem *item, QWidget *parent)
    : NetWidget(item, parent)
    , m_switchBut(new DSwitchButton(this))
{
    QWidget *widget = new QWidget(this);
    widget->setFixedHeight(46);
    QHBoxLayout *layout = new QHBoxLayout(widget);
    layout->setContentsMargins(10, 0, 0, 0);
    DLabel *label = new DLabel(item->name(), this);
    label->setFixedWidth(MAX_TEXT_WIDTH);
    label->setElideMode(Qt::ElideRight);
    label->setForegroundRole(foregroundRole());
    layout->addWidget(label);
    layout->addStretch();
    switch (item->itemType()) {
    case NetItemType::WiredControlItem:
    case NetItemType::WirelessControlItem:
        DFontSizeManager::instance()->bind(label, DFontSizeManager::T4);
        break;
    case NetItemType::WiredDeviceItem:
        DFontSizeManager::instance()->bind(label, DFontSizeManager::T5);
        break;
    case NetItemType::WirelessDeviceItem: {
        DFontSizeManager::instance()->bind(label, DFontSizeManager::T5);
        NetIconButton *refreshBut = new NetIconButton(this);
        refreshBut->setFixedSize(24, 24);
        refreshBut->setIcon(QIcon::fromTheme("refresh"));
        refreshBut->setClickable(true);
        refreshBut->setRotatable(true);
        refreshBut->setVisible(item->isEnabled());
        layout->addWidget(refreshBut);
        connect(refreshBut, &NetIconButton::clicked, this, &NetDeviceWidget::onScanClicked);
        connect(item, &NetDeviceItem::enabledChanged, refreshBut, &NetIconButton::setVisible);
    } break;
    default:
        break;
    }

    layout->addWidget(m_switchBut);
    setCentralWidget(widget);
    onEnabledChanged(item->isEnabled());
    m_switchBut->setEnabled(item->enabledable());
    connect(item, &NetDeviceItem::nameChanged, label, &DLabel::setText);
    connect(item, &NetDeviceItem::enabledChanged, this, &NetDeviceWidget::onEnabledChanged);
    connect(item, &NetDeviceItem::enabledableChanged, m_switchBut, &DSwitchButton::setEnabled);
    connect(m_switchBut, &DSwitchButton::checkedChanged, this, &NetDeviceWidget::onCheckedChanged);
}

NetDeviceWidget::~NetDeviceWidget() { }

void NetDeviceWidget::onEnabledChanged(bool enabled)
{
    m_switchBut->blockSignals(true);
    m_switchBut->setChecked(enabled);
    m_switchBut->blockSignals(false);
}

void NetDeviceWidget::onCheckedChanged(bool checked)
{
    onEnabledChanged(qobject_cast<NetControlItem *>(item())->isEnabled());
    Q_EMIT requestSetEnabled(item()->id(), checked);
}

void NetDeviceWidget::onScanClicked()
{
    Q_EMIT requestScan(item()->id());
}

NetWirelessWidget::NetWirelessWidget(NetWirelessItem *item, QWidget *parent)
    : NetWidget(item, parent)
    , m_isWifi6(item->flags())
    , m_secureBut(new NetIconButton(this))
    , m_iconBut(new NetIconButton(this))
    , m_connBut(new NetIconButton(this))
    , m_loading(new DSpinner(this))
{
    QWidget *widget = new QWidget(this);
    widget->setFixedHeight(36);
    QHBoxLayout *layout = new QHBoxLayout(widget);
    layout->setContentsMargins(3, 0, 5, 0);

    m_secureBut->setFixedSize(20, 20);
    layout->addWidget(m_secureBut);

    m_iconBut->setFixedSize(20, 20);
    layout->addWidget(m_iconBut);
    layout->addSpacing(3);

    DLabel *label = new DLabel(item->name(), this);
    label->setFixedWidth(MAX_TEXT_WIDTH);
    label->setElideMode(Qt::ElideRight);
    label->setForegroundRole(foregroundRole());
    layout->addWidget(label);
    layout->addStretch();
    DFontSizeManager::instance()->bind(label, DFontSizeManager::T6);
    connect(item, &NetDeviceItem::nameChanged, label, &DLabel::setText);

    m_connBut->setIcon(QIcon::fromTheme("select"));
    m_connBut->setHoverIcon(QIcon::fromTheme("disconnect"));
    m_connBut->setForegroundRole(QPalette::Highlight);
    m_connBut->setFixedSize(16, 16);
    m_connBut->setClickable(true);
    layout->addWidget(m_connBut);

    m_loading->setFixedSize(24, 24);
    layout->addWidget(m_loading);
    setCentralWidget(widget);

    onSecureChanged(item->isSecure());
    onStrengthChanged(item->strength());
    onStatusChanged(item->status());

    connect(item, &NetWirelessItem::secureChanged, this, &NetWirelessWidget::onSecureChanged);
    connect(item, &NetWirelessItem::strengthChanged, this, &NetWirelessWidget::onStrengthChanged);
    connect(item, &NetWirelessItem::statusChanged, this, &NetWirelessWidget::onStatusChanged);
    connect(m_connBut, &NetIconButton::clicked, this, &NetWirelessWidget::onDisconnectClicked);
}

NetWirelessWidget::~NetWirelessWidget() { }

void NetWirelessWidget::onStrengthChanged(int strength)
{
    m_iconBut->setIcon(QIcon::fromTheme(QString("wireless%1-%2-symbolic").arg(m_isWifi6 ? "6" : "").arg(NetManager::StrengthLevel(strength)))); // WirelessItem::getStrengthStateString(strength))));
}

void NetWirelessWidget::onSecureChanged(bool secured)
{
    m_secureBut->setIcon(secured ? QIcon::fromTheme("security") : QIcon());
}

void NetWirelessWidget::onStatusChanged(NetConnectionStatus status)
{
    switch (status) {
    case NetConnectionStatus::Connected:
        m_connBut->setVisible(true);
        m_loading->stop();
        m_loading->setVisible(false);
        break;
    case NetConnectionStatus::Connecting:
        m_connBut->setVisible(false);
        m_loading->start();
        m_loading->setVisible(true);
        break;
    default:
        m_connBut->setVisible(false);
        m_loading->stop();
        m_loading->setVisible(false);
        break;
    }
}

void NetWirelessWidget::onDisconnectClicked()
{
    Q_EMIT requestDisconnect(item()->id());
}

NetWirelessHiddenWidget::NetWirelessHiddenWidget(NetWirelessHiddenItem *item, QWidget *parent)
    : NetWidget(item, parent)
{
    QWidget *widget = new QWidget(this);
    widget->setFixedHeight(36);
    QHBoxLayout *layout = new QHBoxLayout(widget);
    layout->setContentsMargins(10, 0, 0, 0);

    DLabel *label = new DLabel(item->name(), this);
    label->setFixedWidth(MAX_TEXT_WIDTH);
    label->setElideMode(Qt::ElideRight);
    DFontSizeManager::instance()->bind(label, DFontSizeManager::T6);
    label->setForegroundRole(foregroundRole());
    layout->addWidget(label);
    layout->addStretch();
    setCentralWidget(widget);
    connect(item, &NetDeviceItem::nameChanged, label, &DLabel::setText);
}

NetWirelessHiddenWidget::~NetWirelessHiddenWidget() { }

NetWiredWidget::NetWiredWidget(NetWiredItem *item, QWidget *parent)
    : NetWidget(item, parent)
    , m_iconBut(new NetIconButton(this))
    , m_connBut(new NetIconButton(this))
    , m_loading(new DSpinner(this))
{
    QWidget *widget = new QWidget(this);
    widget->setFixedHeight(36);
    QHBoxLayout *layout = new QHBoxLayout(widget);
    layout->setContentsMargins(28, 0, 5, 0);

    m_iconBut->setIcon(QIcon::fromTheme("network-wired-symbolic"));
    m_iconBut->setFixedSize(20, 20);
    layout->addWidget(m_iconBut);
    layout->addSpacing(3);

    DLabel *label = new DLabel(item->name(), this);
    label->setFixedWidth(MAX_TEXT_WIDTH);
    label->setElideMode(Qt::ElideRight);
    DFontSizeManager::instance()->bind(label, DFontSizeManager::T6);
    label->setForegroundRole(foregroundRole());
    layout->addWidget(label);
    layout->addStretch();

    m_connBut->setIcon(QIcon::fromTheme("select"));
    m_connBut->setHoverIcon(QIcon::fromTheme("disconnect"));
    m_connBut->setForegroundRole(QPalette::Highlight);
    m_connBut->setFixedSize(16, 16);
    m_connBut->setClickable(true);
    layout->addWidget(m_connBut);

    m_loading->setFixedSize(24, 24);
    layout->addWidget(m_loading);
    setCentralWidget(widget);

    onStatusChanged(item->status());

    connect(item, &NetDeviceItem::nameChanged, label, &DLabel::setText);
    connect(item, &NetWiredItem::statusChanged, this, &NetWiredWidget::onStatusChanged);
    connect(m_connBut, &NetIconButton::clicked, this, &NetWiredWidget::onDisconnectClicked);
}

NetWiredWidget::~NetWiredWidget() { }

void NetWiredWidget::onStatusChanged(NetConnectionStatus status)
{
    switch (status) {
    case NetConnectionStatus::Connected:
        m_connBut->setVisible(true);
        m_loading->stop();
        m_loading->setVisible(false);
        m_iconBut->setIcon(QIcon::fromTheme("network-wired-symbolic"));
        break;
    case NetConnectionStatus::Connecting:
        m_connBut->setVisible(false);
        m_loading->start();
        m_loading->setVisible(true);
        m_iconBut->setIcon(QIcon::fromTheme("network-none-symbolic"));
        break;
    default:
        m_iconBut->setIcon(QIcon::fromTheme("network-none-symbolic"));
        m_connBut->setVisible(false);
        m_loading->stop();
        m_loading->setVisible(false);
        break;
    }
}

void NetWiredWidget::onDisconnectClicked()
{
    Q_EMIT requestDisconnect(item()->id());
}

} // namespace network
} // namespace dde
