// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "connectivitychecker.h"

#include "netutils.h"
#include "netinterface.h"
#include "networkdevicebase.h"
#include "wireddevice.h"
#include "wirelessdevice.h"
#include "configsetting.h"

#include <QEventLoop>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QTimer>

#include <QNetworkRequest>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QDesktopServices>

//当没有进行配置的时候, 则访问我们官网
static const QStringList CheckUrls {
    "https://www.uniontech.com",
};

#define TIMEOUT (3 * 1000)         // 3s超时

using namespace dde::network;

ConnectivityChecker::ConnectivityChecker(QObject *parent)
    : QObject(parent)
{
}

ConnectivityChecker::~ConnectivityChecker()
{
}

void ConnectivityChecker::setCheckPortal(bool checkPortal)
{
    Q_UNUSED(checkPortal);
}

// 通过本地检测网络连通性
LocalConnectionvityChecker::LocalConnectionvityChecker(NetworkProcesser *networkProcesser, QObject *parent)
    : ConnectivityChecker(parent)
    , m_networkProcesser(networkProcesser)
    , m_connectivityChecker(new ConnectivityDetection)
    , m_checkThread(new QThread)
    , m_checkTimer(new QTimer)
    , m_timer(new QTimer)
    , m_checkPortal(false)
    , m_connectivity(Connectivity::Unknownconnectivity)
{
    createConnectivityChecker();
    initConnectivityChecker();
    connect(ConfigSetting::instance(), &ConfigSetting::connectivityCheckIntervalChanged, this, [ = ](int interval) {
        m_checkTimer->setInterval(interval);
    });
}

LocalConnectionvityChecker::~LocalConnectionvityChecker()
{
    for (QMetaObject::Connection connection : m_checkerConnection) {
        disconnect(connection);
    }
    m_checkerConnection.clear();

    m_checkTimer->stop();
    m_checkTimer->deleteLater();
    m_checkTimer = nullptr;

    if (m_timer->isActive())
        m_timer->stop();
    m_timer->deleteLater();
    m_timer = nullptr;

    m_checkThread->quit();
    m_checkThread->wait();
    m_checkThread->deleteLater();
    m_checkThread = nullptr;

    delete m_connectivityChecker;
    m_connectivityChecker = nullptr;
}

void LocalConnectionvityChecker::setCheckPortal(bool checkPortal)
{
    m_checkPortal = checkPortal;
}

Connectivity LocalConnectionvityChecker::connectivity() const
{
    return m_connectivity;
}

void LocalConnectionvityChecker::initDeviceConnect(const QList<NetworkDeviceBase *> &devices)
{
    for (NetworkDeviceBase *device : devices) {
        m_checkerConnection << connect(device, &NetworkDeviceBase::deviceStatusChanged, this, &LocalConnectionvityChecker::onDeviceStatusChanged);
        m_checkerConnection << connect(device, &NetworkDeviceBase::deviceStatusChanged, m_connectivityChecker, &ConnectivityDetection::startCheck, Qt::UniqueConnection);
        m_checkerConnection << connect(device, &NetworkDeviceBase::activeConnectionChanged, this, [ this ] {
            if (!m_timer->isActive())
                m_timer->start();
        }, Qt::UniqueConnection);
        if (device->deviceType() == DeviceType::Wireless) {
            dde::network::WirelessDevice *wirelessDevice = static_cast<dde::network::WirelessDevice *>(device);
            m_checkerConnection << connect(wirelessDevice, &dde::network::WirelessDevice::networkAdded, this, &LocalConnectionvityChecker::onUpdateAccessPointConnect);
            onUpdateAccessPointConnect(wirelessDevice->accessPointItems());
        }
    }
}

void LocalConnectionvityChecker::createConnectivityChecker()
{
    m_checkerConnection << connect(m_connectivityChecker, &ConnectivityDetection::checkFinished, this, &LocalConnectionvityChecker::onCheckStatusChanged);

    // 检查网络的线程
    m_connectivityChecker->moveToThread(m_checkThread);
    m_checkThread->start();
}

void LocalConnectionvityChecker::initConnectivityChecker()
{
    // 定期检查网络的状态
    m_checkTimer->setInterval(ConfigSetting::instance()->connectivityCheckInterval());
    m_checkerConnection << connect(m_checkTimer, &QTimer::timeout, m_connectivityChecker, &ConnectivityDetection::startCheck, Qt::UniqueConnection);
    m_checkTimer->start();

    // 每次执行一次
    m_timer->setInterval(1000);
    m_checkerConnection << connect(m_timer, &QTimer::timeout, m_connectivityChecker, &ConnectivityDetection::startCheck, Qt::UniqueConnection);

    // 当网络设备状态发生变化的时候需要检查网络状态
    initDeviceConnect(m_networkProcesser->devices());

    m_checkerConnection << connect(m_networkProcesser, &NetworkProcesser::deviceAdded, this, [ this ](QList<NetworkDeviceBase*> newDevices) {
        initDeviceConnect(newDevices);
    });
    // 第一次进来的时候执行一次网络状态的检测
    QMetaObject::invokeMethod(m_connectivityChecker, &ConnectivityDetection::startCheck, Qt::QueuedConnection);
}

void LocalConnectionvityChecker::onUpdateAccessPointConnect(const QList<AccessPoints *> accessPoints)
{
    for (dde::network::AccessPoints *accessPoint : accessPoints) {
        m_checkerConnection << connect(accessPoint, &dde::network::AccessPoints::connectionStatusChanged, this, [ this ] {
            if (!m_timer->isActive())
                m_timer->start();
        }, Qt::UniqueConnection);
    }
}

static QSettings *localConfig()
{
    static QSettings localSetting(QDir(QStandardPaths::standardLocations(QStandardPaths::CacheLocation)[0]).path() + "/network.ini", QSettings::IniFormat);
    return &localSetting;
}

void LocalConnectionvityChecker::onDeviceStatusChanged(const DeviceStatus &deviceStatus)
{
    // 如果是网络断开的情况下，需要重新将配置文件里面的配置改回来，方便下次连接的时候可以正常检测
    if (deviceStatus != DeviceStatus::Disconnected)
        return;

    static QSettings *localSetting = localConfig();
    // 只要是设备断开了连接，就让下次进来重新连接的时候直接打开portal认证
    localSetting->setValue("portal/needPortal", true);
}

void LocalConnectionvityChecker::checkUnConnection()
{
    // 设置连接受限,在后面的检测中会更新连接状态
    setConnectivity(Connectivity::Limited);
    if (!m_checkPortal || !ConfigSetting::instance()->checkPortal())
        return;

    // 在网络没有连接的情况下做如下的检测
    static const QString checkUrl("http://detectportal.deepin.com");
    QNetworkRequest request;
    request.setUrl(checkUrl);
    static QNetworkAccessManager manager;
    // 发送请求
    QNetworkReply *reply = manager.get(request);
    // 连接信号和函数槽，处理重定向
    connect(reply, &QNetworkReply::finished, this, [ reply, this ] {
       static QSettings *localSetting = localConfig();
       if (reply->error() != QNetworkReply::NoError) {
           PRINT_DEBUG_MESSAGE << "get reply error:" << reply->errorString();
           reply->deleteLater();
           // 如果进入了这个逻辑,说明不是需要认证,那么下次进来的时候(一般是设备状态发生变化)需要再次打开认证的网页
           localSetting->setValue("portal/needPortal", true);
           return;
       }
       // 获取重定向的网址
       QVariant redirectUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
       if (!redirectUrl.isValid()) {
           PRINT_DEBUG_MESSAGE << "no redirectUrl:" << reply->readAll();
           reply->deleteLater();
           // 这里表示没有认证的页面,一般情况下,进入到这里都表示逻辑是正常的,那么在下次的时候仍然需要认证打开的网页
           localSetting->setValue("portal/needPortal", true);
           return;
       }

       reply->deleteLater();
       setConnectivity(Connectivity::Portal);
       // 如果需要打开认证的网页,那么就直接打开即可
       if (localSetting->value("portal/needPortal", true).toBool()) {
           const QString url = redirectUrl.toString();
           PRINT_INFO_MESSAGE << "open url:" << url;
           // 下次进来的时候无需再次打开网页,因此,此处设置为false
           localSetting->setValue("portal/needPortal", false);
           QDesktopServices::openUrl(url);
       }
    });
}

void LocalConnectionvityChecker::setConnectivity(const Connectivity &connectivity)
{
    if (m_connectivity == connectivity)
        return;

    m_connectivity = connectivity;
    // 更新每个设备的联通性
    emit connectivityChanged(m_connectivity);
}

void LocalConnectionvityChecker::onCheckStatusChanged(bool isConnected)
{
    if (m_timer->isActive())
        m_timer->stop();

    if (isConnected) {
        // 如果检测到了网络是联通的，则认为当前网络为联通
        setConnectivity(Connectivity::Full);
    } else {
        checkUnConnection();
    }
}

// 通过使用NM的方式来获取网络连通性
NMConnectionvityChecker::NMConnectionvityChecker(NetworkProcesser *networkProcesser, QObject *parent)
    : ConnectivityChecker(parent)
    , m_networkProcesser(networkProcesser)
{
    connect(m_networkProcesser, &NetworkProcesser::connectivityChanged, this, &NMConnectionvityChecker::connectivityChanged);
}

NMConnectionvityChecker::~NMConnectionvityChecker()
{
}

Connectivity NMConnectionvityChecker::connectivity() const
{
    return m_networkProcesser->connectivity();
}

// 本地检测网络的连通性
ConnectivityDetection::ConnectivityDetection(QObject *parent)
    : QObject(parent)
    , m_isRunning(false)
{
    connect(ConfigSetting::instance(), &ConfigSetting::checkUrlsChanged, this, &ConnectivityDetection::onUpdateUrls);
    onUpdateUrls(ConfigSetting::instance()->networkCheckerUrls());
}

void ConnectivityDetection::startCheck()
{
    if (m_isRunning) {
        return;
    }

    m_isRunning = true;
    QNetworkAccessManager networkAccessManager;
    for (const QString &url : m_checkUrls) {
        QScopedPointer<QNetworkReply> reply(networkAccessManager.head(QNetworkRequest(QUrl(url))));
        PRINT_DEBUG_MESSAGE << "Check connectivity using url:" << url;

        // Do not use waitForReadyRead to block thread,
        // the QNetworkReply is not implement this virtual method
        // and it will just return false immediately
        // reply->waitForReadyRead(-1);

        // Blocking, about 5 second to timeout
        QTimer timer;
        timer.setSingleShot(true);
        QEventLoop synchronous;
        connect(&timer, &QTimer::timeout, &synchronous, &QEventLoop::quit);
        connect(&networkAccessManager, &QNetworkAccessManager::finished, &synchronous, &QEventLoop::quit);
        timer.start(TIMEOUT);
        synchronous.exec();

        reply->close();
        // 网络状态码中, 大于等于200, 小于等于206的都是网络正常
        if (timer.isActive()) {
            timer.stop();
            if (reply->error() == QNetworkReply::NoError &&
                    (reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() >= 200 &&
                    reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() <= 206)) {
                PRINT_DEBUG_MESSAGE << "Connected to url:" << url;
                Q_EMIT checkFinished(true);
                m_isRunning = false;
                return;
            }
        } else {
            PRINT_DEBUG_MESSAGE << "Timeout";
        }
    }

    Q_EMIT checkFinished(false);
    m_isRunning = false;
}

void ConnectivityDetection::onUpdateUrls(const QStringList &urls)
{
    m_checkUrls = urls;
    if (m_checkUrls.isEmpty())
        m_checkUrls = CheckUrls;
}
