// SPDX-FileCopyrightText: 2018 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#include "connectivityprocesser.h"
#include "connectivitychecker.h"
#include "netinterface.h"
#include "wirelessdevice.h"

#include "networkdevicebase.h"
#include "netutils.h"
#include "configsetting.h"

using namespace dde::network;

ConnectivityProcesser::ConnectivityProcesser(NetworkProcesser *networkProcesser, QObject *parent)
    : QObject(parent)
    , m_networkProcesser(networkProcesser)
{
    ConfigSetting *configSetting = ConfigSetting::instance();
    connect(configSetting, &ConfigSetting::enableConnectivityChanged, this, &ConnectivityProcesser::onEnableConnectivityChanged);
    m_checker.reset(createConnectivityChecker(configSetting->enableConnectivity()));
}

ConnectivityProcesser::~ConnectivityProcesser()
{
}

Connectivity ConnectivityProcesser::connectivity() const
{
    return m_checker->connectivity();
}

void ConnectivityProcesser::setCheckPortal(bool checkPortal)
{
    m_checker->setCheckPortal(checkPortal);
}

ConnectivityChecker *ConnectivityProcesser::createConnectivityChecker(bool enableConnectivity)
{
    ConnectivityChecker *checker = nullptr;
    if (enableConnectivity) {
        PRINT_DEBUG_MESSAGE << "uses local connectivity checker";
        checker = new LocalConnectionvityChecker(m_networkProcesser, this);
    } else {
        PRINT_DEBUG_MESSAGE << "uses nm connectivity checker";
        checker = new NMConnectionvityChecker(m_networkProcesser, this);
    }

    connect(checker, &ConnectivityChecker::connectivityChanged, this, &ConnectivityProcesser::onConnectivityChanged);

    return checker;
}

void ConnectivityProcesser::onEnableConnectivityChanged(bool enableConnectivity)
{
    m_checker.reset(createConnectivityChecker(enableConnectivity));
}

void ConnectivityProcesser::onConnectivityChanged(const Connectivity &connectivity)
{
    QList<NetworkDeviceBase *> devices = m_networkProcesser->devices();
    for (NetworkDeviceBase *device : devices) {
        device->deviceRealize()->setConnectivity(connectivity);
    }
    emit connectivityChanged(connectivity);
}
