#include "activeconnectioninfo.h"
#include "devicemanagerrealize.h"

#include <NetworkManagerQt/Manager>
#include <NetworkManagerQt/WirelessDevice>
#include <NetworkManagerQt/AccessPoint>
#include <NetworkManagerQt/WirelessSecuritySetting>
#include <NetworkManagerQt/Security8021xSetting>
#include <NetworkManagerQt/VpnSetting>

#include <QJsonArray>

using namespace dde::network;

#define ServiceTypeL2TP         "org.freedesktop.NetworkManager.l2tp"
#define ServiceTypePPTP         "org.freedesktop.NetworkManager.pptp"
#define ServiceTypeVPNC         "org.freedesktop.NetworkManager.vpnc"
#define ServiceTypeOpenVPN      "org.freedesktop.NetworkManager.openvpn"
#define ServiceTypeStrongSwan   "org.freedesktop.NetworkManager.strongswan"
#define ServiceTypeOpenConnect  "org.freedesktop.NetworkManager.openconnect"
#define ServiceTypeSSTP         "org.freedesktop.NetworkManager.sstp"

ActiveConnectionInfo::ActiveConnectionInfo(QObject *parent)
    : QObject(parent)
{
}

QJsonArray ActiveConnectionInfo::json(const QStringList &allDevicePaths) const
{
    QJsonArray array;
    NetworkManager::ActiveConnection::List activeConns = NetworkManager::activeConnections();
    for (NetworkManager::ActiveConnection::Ptr activeConn : activeConns) {
        if (activeConn->state() != NetworkManager::ActiveConnection::State::Activated)
            continue;

        // 获取当前连接的所有的设备的信息
        QStringList devicePaths = activeConn->devices();
        for (const QString &devicePath : devicePaths) {
            if (!allDevicePaths.contains(devicePath))
                continue;
            // 遍历每个设备的活动连接
            QJsonObject jsonObject = json(devicePath, activeConn);
            if (jsonObject.isEmpty())
                continue;

            array << jsonObject;
        }
    }

    return array;
}

static QString connectionTypeAsString(const NetworkManager::Device::Ptr &device, const NetworkManager::ConnectionSettings::Ptr &settings)
{
    NetworkManager::ConnectionSettings::ConnectionType type = settings->connectionType();
    if (type == NetworkManager::ConnectionSettings::ConnectionType::Wired)
        return "wired";

    if (type == NetworkManager::ConnectionSettings::ConnectionType::Wireless) {
        NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
        // 如果当前连接开启了热点，则无需填入IP
        if (wirelessDevice) {
            switch (wirelessDevice->mode()) {
            case NetworkManager::WirelessDevice::OperationMode::ApMode:
                return "wireless-hotspot";
            case NetworkManager::WirelessDevice::OperationMode::Adhoc:
                return "wireless-adhoc";
            default:
                break;
            }
        }
        return "wireless";
    }

    switch (type) {
    case NetworkManager::ConnectionSettings::ConnectionType::Pppoe:
        return "pppoe";
    case NetworkManager::ConnectionSettings::ConnectionType::Gsm:
        return "mobile-gsm";
    case NetworkManager::ConnectionSettings::ConnectionType::Cdma:
        return "mobile-cdma";
    case NetworkManager::ConnectionSettings::ConnectionType::Vpn: {
        const QString &serviceType = settings->setting(NetworkManager::Setting::SettingType::Vpn).staticCast<NetworkManager::VpnSetting>()->serviceType();
        if (serviceType == ServiceTypeL2TP)
            return "vpn-l2tp";
        if (serviceType == ServiceTypePPTP)
            return "vpn-pptp";
        if (serviceType == ServiceTypeOpenConnect)
            return "vpn-openconnect";
        if (serviceType == ServiceTypeOpenVPN)
            return "vpn-openvpn";
        if (serviceType == ServiceTypeStrongSwan)
            return "vpn-strongswan";
        if (serviceType == ServiceTypeVPNC)
            return "vpn-vpnc";
        if (serviceType == ServiceTypeSSTP)
            return "vpn-sstp";
        break;
    }
    default:
        break;
    }

    return "unknow";
}

QString ActiveConnectionInfo::getSecurity(const NetworkManager::Device::Ptr &device) const
{
    NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
    if (wirelessDevice.isNull())
        return QString();

    NetworkManager::AccessPoint::Ptr activeAccessPoint = wirelessDevice->activeAccessPoint();
    NetworkManager::ActiveConnection::Ptr activeConnection = wirelessDevice->activeConnection();
    if (activeConnection.isNull() || activeAccessPoint.isNull())
        return QString();

    NetworkManager::ConnectionSettings::ConnectionType connectionType = activeConnection->connection()->settings()->connectionType();
    if (connectionType != NetworkManager::ConnectionSettings::ConnectionType::Wireless || activeAccessPoint.isNull())
        return QString();

    NetworkManager::WirelessSecuritySetting::KeyMgmt keyMemt;
    NetworkManager::WirelessSecuritySetting::Ptr securitySetting = activeConnection->connection()->settings()->setting(
                NetworkManager::Setting::SettingType::WirelessSecurity).dynamicCast<NetworkManager::WirelessSecuritySetting>();
    if (securitySetting.isNull()) {
        keyMemt = WirelessDeviceManagerRealize::getKeyMgmtByAp(activeAccessPoint);
    } else {
        keyMemt = securitySetting->keyMgmt();
    }
    QString security = QObject::tr("None");
    switch (keyMemt) {
    case NetworkManager::WirelessSecuritySetting::KeyMgmt::Wep:
        security = QObject::tr("WEP 40/128-bit Key");
        break;
    case NetworkManager::WirelessSecuritySetting::KeyMgmt::WpaPsk:
        security = QObject::tr("WPA/WPA2 Personal");
        break;
    case NetworkManager::WirelessSecuritySetting::KeyMgmt::WpaSae:
        security = QObject::tr("WPA3 Personal");
        break;
    case NetworkManager::WirelessSecuritySetting::KeyMgmt::WpaEap: {
        NetworkManager::Security8021xSetting::Ptr wirelessSecuritySetting = activeConnection->connection()->settings()->setting(NetworkManager::Setting::SettingType::Security8021x).dynamicCast<NetworkManager::Security8021xSetting>();
        if (!wirelessSecuritySetting.isNull()) {
            const QList<NetworkManager::Security8021xSetting::EapMethod> &eapMethods = wirelessSecuritySetting->eapMethods();
            NetworkManager::Security8021xSetting::EapMethod currentEapMethod = eapMethods.isEmpty() ? NetworkManager::Security8021xSetting::EapMethodTls : eapMethods.first();
            switch (currentEapMethod) {
            case NetworkManager::Security8021xSetting::EapMethodTls:
                security = "EAP" + QObject::tr("TLS");
                break;
            case NetworkManager::Security8021xSetting::EapMethodMd5:
                security = "EAP/" + QObject::tr("MD5");
                break;
            case NetworkManager::Security8021xSetting::EapMethodLeap:
                security = "EAP/" + QObject::tr("LEAP");
                break;
            case NetworkManager::Security8021xSetting::EapMethodFast:
                security = "EAP/" + QObject::tr("FAST");
                break;
            case NetworkManager::Security8021xSetting::EapMethodTtls:
                security = "EAP/" + QObject::tr("Tunneled TLS");
                break;
            case NetworkManager::Security8021xSetting::EapMethodPeap:
                security = "EAP/" + QObject::tr("Protected EAP");
                break;
            default:
                break;
            }
        }
        break;
    }
    default:
        break;
    }
    return security;
}

const QString ActiveConnectionInfo::macAddress(const NetworkManager::Device::Ptr &device) const
{
    switch (device->type()) {
    case NetworkManager::Device::Type::Ethernet: {
        NetworkManager::WiredDevice::Ptr wiredDevice = device.dynamicCast<NetworkManager::WiredDevice>();
        if (wiredDevice)
            return wiredDevice->hardwareAddress();
        break;
    }
    case NetworkManager::Device::Type::Wifi: {
        NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
        if (wirelessDevice)
            return wirelessDevice->hardwareAddress();
        break;
    }
    default:
        break;
    }

    return QString();
}

QJsonObject ActiveConnectionInfo::json(const QString &devicePath, const NetworkManager::ActiveConnection::Ptr &activeConn) const
{
    NetworkManager::Device::Ptr device = NetworkManager::findNetworkInterface(devicePath);
    if (device.isNull() || activeConn.isNull())
        return QJsonObject();

    NetworkManager::Connection::Ptr connection = activeConn->connection();

    const QString type = connectionTypeAsString(device, activeConn->connection()->settings());
    QJsonObject activeConnJson;
    NetworkManager::ActiveConnection::Ptr primaryConnection = NetworkManager::primaryConnection();
    activeConnJson.insert("IsPrimaryConnection", (primaryConnection && primaryConnection->path() == activeConn->path()));
    activeConnJson.insert("Device", devicePath);
    activeConnJson.insert("SettingPath", connection ? connection->path() : QString());
    activeConnJson.insert("SpecificObject", activeConn->specificObject());
    activeConnJson.insert("ConnectionType", type);
    activeConnJson.insert("Protocol", NetworkManager::ConnectionSettings::typeAsString(connection->settings()->connectionType()));
    activeConnJson.insert("ConnectionName", connection->name());
    activeConnJson.insert("ConnectionUuid", connection->uuid());
    activeConnJson.insert("Security", getSecurity(device));
    activeConnJson.insert("DeviceType", type);
    activeConnJson.insert("DeviceInterface", device->interfaceName());
    activeConnJson.insert("HwAddress", macAddress(device));
    activeConnJson.insert("Hotspot", hotspotJson(device));
    switch (device->type()) {
    case NetworkManager::Device::Ethernet: {
        uint speed = 0;
        NetworkManager::WiredDevice::Ptr wiredDevice = device.dynamicCast<NetworkManager::WiredDevice>();
        if (wiredDevice) {
            speed = static_cast<uint>(wiredDevice->bitRate() / 1000);
        }
        activeConnJson.insert("Speed", QString("%1 Mbps").arg(speed));
        break;
    }
    case NetworkManager::Device::Wifi: {
        uint speed = 0;
        NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
        if (wirelessDevice) {
            speed = static_cast<uint>(wirelessDevice->bitRate() / 1000);
        }
        activeConnJson.insert("Speed", QString("%1 Mbps").arg(speed));
        break;
    }
    default:
        activeConnJson.insert("Speed", QString("0 Mbps"));
        break;
    }

    QJsonObject ipv4Object = ipJson(device, activeConn);
    if (!ipv4Object.isEmpty())
        activeConnJson.insert("IPv4", ipv4Object);

    QJsonObject ipv6Object = ipJson(device, activeConn, true);
    if (!ipv6Object.isEmpty())
        activeConnJson.insert("IPv6", ipv6Object);

    return activeConnJson;
}

QJsonObject ActiveConnectionInfo::hotspotJson(const NetworkManager::Device::Ptr &device) const
{
    QString ssid;
    QString band;
    int channel = 0;
    do {
        if (device->type() != NetworkManager::Device::Type::Wifi)
            break;
        // 获取设备的活动连接网络
        NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
        if (wirelessDevice.isNull())
            break;

        NetworkManager::AccessPoint::Ptr activeAccessPoint = wirelessDevice->activeAccessPoint();
        if (activeAccessPoint.isNull())
            break;

        ssid = activeAccessPoint->ssid();

        uint frequency = activeAccessPoint->frequency();
        band = "unknown";
        if (frequency >= 4915 && frequency <= 5825)
            band = "a";
        else if (frequency >= 2412 && frequency <= 2484)
            band = "bg";

        static QMap<uint, int> frequencyChannelMap {
            {2412, 1}, {2417, 2}, {2422, 3}, {2427, 4}, {2432, 5}, {2437, 6}, {2442, 7},
            {2447, 8}, {2452, 9}, {2457, 10}, {2462, 11}, {2467, 12}, {2472, 13}, {2484, 14},
            {5035, 7}, {5040, 8}, {5045, 9}, {5055, 11}, {5060, 12}, {5080, 16}, {5170, 34},
            {5180, 36}, {5190, 38}, {5200, 40}, {5220, 44}, {5230, 44}, {5240, 48}, {5260, 52}, {5280, 56}, {5300, 60},
            {5320, 64}, {5500, 100}, {5520, 104}, {5540, 108}, {5560, 112}, {5580, 116}, {5600, 120},
            {5620, 124}, {5640, 128}, {5660, 132}, {5680, 136}, {5700, 140}, {5745, 149}, {5765, 153},
            {5785, 157}, {5805, 161}, {5825, 165}, {4915, 183}, {4920, 184}, {4925, 185}, {4935, 187},
            {4940, 188}, {4945, 189}, {4960, 192}, {4980, 196}};
        channel = frequencyChannelMap.value(frequency, 0);
    } while (false);

    QJsonObject object;
    object.insert("Ssid", ssid);
    object.insert("Band", band);
    object.insert("Channel", channel);
    return object;
}

QJsonObject ActiveConnectionInfo::ipJson(const NetworkManager::Device::Ptr &device, const NetworkManager::ActiveConnection::Ptr &activeConnection, bool isIpV6) const
{
    if (device->type() == NetworkManager::Device::Type::Wifi) {
        NetworkManager::WirelessDevice::Ptr wirelessDevice = device.dynamicCast<NetworkManager::WirelessDevice>();
        // 如果当前连接开启了热点，则无需填入IP
        if (!wirelessDevice || wirelessDevice->mode() == NetworkManager::WirelessDevice::OperationMode::ApMode)
            return QJsonObject();
    }

    if (isIpV6) {
        // 获取IpV6的地址
        QJsonObject ipV6Json;
        QJsonArray ipV6Array;
        NetworkManager::IpConfig ipV6Config = activeConnection->ipV6Config();
        QList<NetworkManager::IpAddress> ipV6addresses = ipV6Config.addresses();
        if (ipV6addresses.size() > 0) {
            for (NetworkManager::IpAddress address : ipV6addresses) {
                QJsonObject ipAddr;
                ipAddr.insert("Address", address.ip().toString());
                ipAddr.insert("Prefix", address.prefixLength());
                ipV6Array.append(ipAddr);
            }
        } else {
            QJsonObject ipAddr;
            ipAddr.insert("Address", "0::0");
            ipAddr.insert("Prefix", "0");
            ipV6Array.append(ipAddr);
        }
        ipV6Json.insert("Addresses", ipV6Array);
        ipV6Json.insert("Gateway", ipV6Config.gateway());
        QJsonArray ipV6NameServersArray;
        QList<QHostAddress> ipV6NameServers = ipV6Config.nameservers();
        for (const QHostAddress &nameServer : ipV6NameServers) {
            ipV6NameServersArray.append(nameServer.toString());
        }
        ipV6Json.insert("Nameservers", ipV6NameServersArray);
        return ipV6Json;
    }
    // 获取IPV4的地址
    QJsonObject ipV4Json;
    QJsonArray ipV4Array;
    NetworkManager::IpConfig ipV4Config = activeConnection->ipV4Config();
    QList<NetworkManager::IpAddress> addresses = ipV4Config.addresses();
    for (NetworkManager::IpAddress address : addresses) {
        QJsonObject ipAddr;
        ipAddr.insert("Address", address.ip().toString());
        ipAddr.insert("Prefix", address.prefixLength());
        ipV4Array.append(ipAddr);
    }
    ipV4Json.insert("Addresses", ipV4Array);
    ipV4Json.insert("Gateway", ipV4Config.gateway());
    QJsonArray ipV4NameServersArray;
    QList<QHostAddress> ipV4NameServers = ipV4Config.nameservers();
    for (const QHostAddress &ipV4NameServer : ipV4NameServers) {
        ipV4NameServersArray.append(ipV4NameServer.toString());
    }
    ipV4Json.insert("Nameservers", ipV4NameServersArray);
    return ipV4Json;
}
